#$Id: //depot/prod/test/Rsierranevada/storage/hdd/NADQ_SEA/FS/Common_Lib.pm#39 $
#
#  Library of utility functions for HDD qualification tests
#
#
## @summary Library of routines to help simply HDD qualfication test development
## @description Library of routines to help in simplify the verfication required for various test cases oh hdd test plan.
##
## @synopsis
## use Storage::Common_Lib;
##
##
## @author
##         arunak@netapp.com
####################################################################
##
## Design guidelines to be followed
##
## All subroutine must start with nvme_<subroutine>
## All Subriutine must return has with all details or values in output <wherever itt is applicable>
## Each subroutine must use validate options
## Each subroutine must contain author and usage of subroutines
## Every optional value must contain default value assigned
##
##
###################################################################
package Storage::ZNS_Common_Lib;
use TCD;
use CR::Setup::TCD qw(nlog);
use strict;
use Storage::Common_Lib;
use Storage::NVMe_Common_Lib;
use Storage::Tahiti_Common_Lib;
use Tharn;
use NACL::C::StorageAggregate;
use NACL::APISet;
use Data::Dumper;
use NACL::C::Disk;
use NACL::C::Volume;
use NACL::C::Aggregate;
use NACL::C::Vserver;
use NACL::C::VolumeMember;
use NACL::CS::StorageAggregateMember;
use NACL::MTask::EventLogDetector;
use Utils;
use HTML::Mail;
use NACL::MTask::SCP;
use NACL::C::Client;
use NACL::STask::SecurityLogin;
use NACL::STask::Node;
use NACL::MTask::SystemLogDetector;
use boot;
use San;
use List::Util qw( min max shuffle);
use Utils qw(make_conn);
use Storage::SASUtils qw(filer_cmd);
use Storage::Misc qw(filer_cmd);
use install;
use misc qw(cli_localhost);
use Storage::Storage qw(storage_show_adapter);
use POSIX;
use NACL::Transit;
use Storage::SASUtils qw(disable_dbg_msgs);
use HTML::Mail;
use NACL::C::Node;
use Tharn qw(sleep);
use Tharn qw(param);
use Hostrec;
use Server;
use Options;
use Expect;
use SayRetry;
use mount;
use NACL::MTask::EventLogDetector;
use NACL::STask::StorageFailover;
use NACL::CS::Cluster;
use lib "/usr/PNATE/pnate/lib";
use PNATE;
use lib "/usr/PNATE/nate/lib";
use NATE;

my $params = NATE::ParamSet->new( global => 1 );
my $FILER          = $params->get( 'FILER',          default => undef );
my $Kernel         = $params->get( 'KERNEL',         default => 'SN' );
my $EMAIL          = $params->get( 'EMAIL',          default => undef );
my $MAIL_TO        = $params->get( 'MAIL_TO',        default => 'Email to' );
my $MAIL_FROM      = $params->get( 'MAIL_FROM',      default => 'Email from' );
my $RUNID          = $params->get( 'RUNID',          default => 'undef' );
my $STEST_FILE     = $params->get( 'STEST_FILE',     default => '' );
my $BOOT_MODE      = $params->get( 'BOOT_MODE',      default => undef );
my $Filer_mode     = $params->get( 'Filer_mode',     default => 'F' );
my $TEST_WAIT_TIME = $params->get( 'TEST_WAIT_TIME', default => '3' );
my $GROUP_NO       = $params->get( 'GROUP_NO',       default => undef );
my $DRIVE_TYPE     = $params->get( 'DRIVE_TYPE',     default => '' );
my $ERT_VOL        = $params->get( "ERT_VOL",        default => '1' );
my $LOGDIR         = $params->get( "LOGDIR",         default => 'NA' );
my $BOOT_ARGS_SEL  = $params->get( "BOOT_ARGS_SEL",  default => 'NA' );

my $PRE_DETAILS;
my $POST_DETAILS;
my @pre_disk_count;
my @post_disk_count;
my $pre_post_recon_copy;
my %Non_Medium_Model;

#logcomment("Kernel $Kernel");Commented for burt 1392872
logcomment("Id $RUNID");
my @run_id   = split( '/', $RUNID );
my $parse_id = $run_id[-1];
my $timeout  = 3600;

#LOCAL VAR
my $SCSI_DISK_ERRORS = {};
our $GET_PHY_STATE;
our $STO_DISK_SHOW = {};

my @Filers;
if ( $FILER =~ /\,/ ) {
    @Filers = split( /\,/, $FILER );
} else {
    push( @Filers, $FILER );
}
if ( $Kernel =~ /FS|LB/i ) {
    require NACL::C::StorageDiskPartition;
    require NACL::CS::StorageDiskPartition;
}

# Import exceptions
#
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw (:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MethodNotFoundException qw(:try);
use NACL::Exceptions::NoElementsFound qw(:try);
use NATE::BaseException qw(:try);
use NACL::Transit::Exceptions::Timeout qw(:try);
use NACL::Transit::Exceptions::RetriesExceeded qw(:try);
use NACL::Transit::Exceptions::TransitException qw(:try);
use NACL::APISet::Exceptions::TimeoutException qw(:try);
use NACL::Exceptions::EventCheckFailure qw(:try);
use NACL::APISet::Exceptions::MgwdCrashedException qw(:try);
use NACL::APISet::Exceptions::NoMatchingEntriesException qw(:try);

#use NATE::Result::Fatal qw(:try);
#
use Params::Validate qw(validate OBJECT SCALAR HASHREF ARRAYREF);
use Tharn qw( host logcomment );

BEGIN {
    Params::Validate::validation_options(
        on_fail => sub {
            my @msgs = @_;
            chomp(@msgs);
            Carp::confess( Tharn::script_result(@msgs) );
        }
    );
}

require Exporter;
use vars ( '@ISA', '@EXPORT' );
@ISA = ('Exporter');

@EXPORT = qw(
  zns_eye_catcher
  zns_identify_cmds
  disp_pegnvme_drive_variables
  zns_pre_test
  zns_post_test
  get_pegnvme_from_file
  zns_get_disk_info
  get_zns_systemshell_drvs
  get_zns_identify_active_drv_details
  get_zns_owner_node
  set_zns_boot_args
  zns_assign_drives
  setup_cmode_zns
  wipe_config_zns
  detect_zns_drives
  random_num_gen_min2max
  zns_copy_FW_files_to_filer_directory
  zns_check_drv_path
  READ_SKTRACE
  READ_LINK_STATUS
  READ_SMART
);
############################################################################################
## Boot-Args
#setenv pegasus.nvme.disable_configure_io_cmd_set true
#setenv storage.disable.unqualified.check true
#setenv pegasus.nvme.allow_write_access true
#setenv sdf.ztl_ready_to_go  true
#setenv use-resv?            false
#
## this enables Samsung NT04 only, all other drives will be failed out
#setenv sdf.use_updated_zone_spec2 true
############################################################################################
sub set_zns_boot_args {
    my (%opts) = validate(
        @_,
        {
            node_present => { type => ARRAYREF },
            Handle       => { type => SCALAR, default => undef, optional => 1 },
        }
    );
    my @Nodes  = @{ $opts{node_present} };
    my $handle = $opts{Handle};
    my %nodes_filer;
    my @re_subtest;
    my $count = 0;
    my $Nodes_zns;
    my $Nodes_nonzns;

    foreach my $node (@Nodes) {
        my $Api_Set_Obj = $node->get_7m_or_nodescope_apiset();
        my $prompts_answers = [ ".*Do you want to continue.*" => 'y' ];
        $Api_Set_Obj->execute_raw_command( 'command' => "set test", 'connectrec-match_table' => $prompts_answers );

=head
        my $zout = $Api_Set_Obj->execute_command( 'command' => "disk_list" );
        foreach my $line ( split( /\n/, $zout ) ) {
            if ( $line =~ /(\S+)\s*NVMe\s*(\S+)\s*\S+/ ) {
                $Nodes_zns = $node;
                $count     = 1;
            }
        }
=cut

        my $prompts_answers1 = [ ".*to page down, .*" => ' ' ];
        my $zout = $Api_Set_Obj->execute_raw_command( 'command' => "storage show disk -fields type", 'connectrec-match_table' => $prompts_answers1 );
        foreach my $line ( split( /\n/, $zout ) ) {
            next if ( $line =~ /disk|type/ );
            if ( $line =~ /(\d+\.\d+\.\d+)\s+SSD-ZNS/ ) {
                $Nodes_zns = $node;
                $count     = 1;
            }
        }
        last if ( $count == 1 );
    }

    if ( $count == 1 ) {
        foreach my $node (@Nodes) {
            if ( $node ne $Nodes_zns ) {
                $Nodes_nonzns = $node;
            }
        }
    } else {

        #		logcomment("ZNS drive is not found on filer ");
        #		logresult("**FATAL**: ZNS drive is not found on filer ");
        $Nodes_zns    = @Nodes[0];
        $Nodes_nonzns = @Nodes[1];
    }

    my $FILER_C = $Nodes_zns->name();
    logcomment("Node / filer : $FILER_C ");
    logcomment("Check state of filer $FILER_C");
    my $Transit_obj;
    my $timeout = 6000;
    $Transit_obj = NACL::Transit->new( name => $FILER_C );
    my $filer_state = $Transit_obj->get_state( 'timeout' => $timeout, 'get_state_timeout' => $timeout );
    logcomment("$FILER_C : Filer is in $filer_state ");
    $nodes_filer{$FILER_C}->{"Original_state"} = $filer_state;

    if ( $filer_state !~ /FIRMWARE/ ) {
        logcomment("Changing Filer $FILER_C state to LOADER");
        push(
            @re_subtest,
            subtest(
                \&boot_filer_in_req_state, -runid, "shutdown_node_" . "$FILER_C", -bg, "--",
                filer_name      => $FILER_C,
                change_state_to => 'FIRMWARE'
            )
        );
    } else {
        logcomment("Filers is in LOADER");
    }

    Subtest::wait_finish( subtest => [@re_subtest] );
    logcomment("Booted Filers in FIRMWARE(Loader)");
    $Nodes_zns->refresh_command_interface();
    my $ignore_strings = [ "MADIMM\\d+->", "DIMM\\d+->", "^T>" ];
    my $FILER          = $Nodes_zns->name();
    my $host           = host($FILER);
    my $prompt_array   = $host->prompt_fw();
    my $prompts        = join( "|", @$prompt_array );
    my $prompt_fw      = "($prompts)";
    my $Transit_obj    = NACL::Transit->new( name => $FILER );
    my $state          = $Transit_obj->get_state( 'timeout' => 3600, 'get_state_timeout' => 7200 );

    if ( $state =~ /FIRMWARE/ ) {
        logcomment("FILER is in LOADER prompt, disabling auto partition env variables");
        my $Server = new Server( no_cli => 1 );
        $Server->set_execution_server( -Server => $FILER );
        $Server->console(
            cmd                        => 'setenv ontap.sdf_enabled true',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );

        $Server->console(
            cmd                        => 'setenv pegasus.nvme.allow_write_access true',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv pegasus.nvme.disable_configure_io_cmd_set true',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv sdf.drive_clean_boot false',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv sdf.ztl_ready_to_go  true',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv storage.disable.unqualified.check true',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv use-resv? false',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv ztl.cli.allow_write_access true',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv raid_vol_writeio_res_count_v3 16',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );

=head
        if ( $BOOT_ARGS_SEL == 1 ) {
            my $out = $Server->console(
                cmd                        => 'unsetenv sdf.use_updated_zone_spec2',
                additional_prompts_timeout => 60,
                ignore                     => $ignore_strings,
                additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
            );
if($out =~ /not found/){
logcomment("Environment variable 'sdf.use_updated_zone_spec2' is unset already");
}
 my $out = $Server->console(
                cmd                        => 'unsetenv sdf.use_updated_zone_spec3',
                additional_prompts_timeout => 60,
                ignore                     => $ignore_strings,
                additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
            );
if($out =~ /not found/){
logcomment("Environment variable 'sdf.use_updated_zone_spec3' is unset already");
}

        } elsif ( $BOOT_ARGS_SEL == 2 ) {
            my $out = $Server->console(
                cmd                        => 'unsetenv sdf.use_updated_zone_spec2',
                additional_prompts_timeout => 60,
                ignore                     => $ignore_strings,
                additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
            );
if($out =~ /not found/){
logcomment("Environment variable 'sdf.use_updated_zone_spec2' is unset already");
}

 $Server->console(
                cmd                        => 'setenv sdf.use_updated_zone_spec3 false',
                additional_prompts_timeout => 60,
                ignore                     => $ignore_strings,
                additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
            );

        } else {
            $Server->console(
                cmd                        => 'setenv sdf.use_updated_zone_spec2 false',
                additional_prompts_timeout => 60,
                ignore                     => $ignore_strings,
                additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
            );
 $Server->console(
                cmd                        => 'setenv sdf.use_updated_zone_spec3 false',
                additional_prompts_timeout => 60,
                ignore                     => $ignore_strings,
                additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
            );


        }
=cut

    } else {
        logcomment("Boot arg not set, as filer is not in Loader");
    }
    ##Setting NON_ZNS Node Boot agrs
    my $FILER_C = $Nodes_nonzns->name();
    logcomment("Node / filer : $FILER_C ");
    logcomment("Check state of filer $FILER_C");
    my $Transit_obj;
    my $timeout = 6000;
    $Transit_obj = NACL::Transit->new( name => $FILER_C );
    my $filer_state = $Transit_obj->get_state( 'timeout' => $timeout, 'get_state_timeout' => $timeout );
    logcomment("$FILER_C : Filer is in $filer_state ");
    $nodes_filer{$FILER_C}->{"Original_state"} = $filer_state;

    if ( $filer_state !~ /FIRMWARE/ ) {
        logcomment("Changing Filer $FILER_C state to LOADER");
        push(
            @re_subtest,
            subtest(
                \&boot_filer_in_req_state, -runid, "shutdown_node_" . "$FILER_C", -bg, "--",
                filer_name      => $FILER_C,
                change_state_to => 'FIRMWARE'
            )
        );
    } else {
        logcomment("Filers is in LOADER");
    }
    Subtest::wait_finish( subtest => [@re_subtest] );
    $Nodes_nonzns->refresh_command_interface();
    my $ignore_strings = [ "MADIMM\\d+->", "DIMM\\d+->", "^T>" ];
    my $FILER          = $Nodes_nonzns->name();
    my $host           = host($FILER);
    my $prompt_array   = $host->prompt_fw();
    my $prompts        = join( "|", @$prompt_array );
    my $prompt_fw      = "($prompts)";
    my $Transit_obj    = NACL::Transit->new( name => $FILER );
    my $state          = $Transit_obj->get_state( 'timeout' => 3600, 'get_state_timeout' => 7200 );

    if ( $state =~ /FIRMWARE/ ) {
        logcomment("FILER is in LOADER prompt, disabling auto partition env variables");
        my $Server = new Server( no_cli => 1 );
        $Server->set_execution_server( -Server => $FILER );
        $Server->console(
            cmd                        => 'setenv ontap.sdf_enabled true',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv pegasus.nvme.allow_write_access true',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv pegasus.nvme.disable_configure_io_cmd_set true',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv sdf.drive_clean_boot false',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv sdf.ztl_ready_to_go true',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv storage.disable.unqualified.check true',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv use-resv? false',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv ztl.cli.allow_write_access true',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
        $Server->console(
            cmd                        => 'setenv raid_vol_writeio_res_count_v3 16',
            additional_prompts_timeout => 60,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT", }
        );
    } else {
        logcomment("Boot arg not set, as filer is not in Loader");
    }

    logcomment("Boot args set on both nodes");
    logcomment("Boot filers back to original state");
    my @chnage_s;
    foreach my $node (@Nodes) {
        my $FILER_C = $node->name();
        logcomment("Node / filer : $FILER_C ");
        logcomment("Check state of filer $FILER_C");
        my $Transit_obj;
        my $timeout = 6000;
        $Transit_obj = NACL::Transit->new( name => $FILER_C );
        my $filer_state = $Transit_obj->get_state( 'timeout' => $timeout, 'get_state_timeout' => $timeout );
        logcomment("$FILER_C : Filer current state is $filer_state ");
        my $boot_state = $nodes_filer{$FILER_C}->{"Original_state"};
        logcomment("$FILER_C : original State was $boot_state, Change state to $boot_state");
        push(
            @chnage_s,
            subtest(
                \&boot_filer_in_req_state, -runid, "shutdown_node_" . "$FILER_C", -bg, "--",
                filer_name      => $FILER_C,
                change_state_to => $boot_state,
            )
        );
    }
    Subtest::wait_finish( subtest => [@chnage_s] );
    logcomment("Booted back to 'Original_state'");
}

#########################################################
# Description : Assign all ZNS drives to ZNS node
# Return 1 if assigned or 0 if not assigned
#########################################################
sub zns_assign_drives {
    my (%opts) = validate( @_, { node_present => { type => ARRAYREF }, } );
    my @Nodes = @{ $opts{node_present} };
    my @zns_drives = ();
    my $count      = 0;
    my $FILER_C;
    my $Api_Set_Obj;
    my $NODE_A_CON;
    my $NODE_B_CON;
    my $FILER_A;
    my $FILER_B;
    my $Api_Set_Obj_A;
    my $Api_Set_Obj_B;

    foreach my $node (@Nodes) {
        $Api_Set_Obj = $node->get_7m_or_nodescope_apiset();
        my $prompts_answers = [ ".*Do you want to continue.*" => 'y' ];
        $Api_Set_Obj->execute_raw_command( 'command' => "set test", 'connectrec-match_table' => $prompts_answers );

=head
        my $zout = $Api_Set_Obj->execute_raw_command( 'command' => "run local disk_list" );
        foreach my $line ( split( /\n/, $zout ) ) {
            if ( $line =~ /(\S+)\s*NVMe\s*(\S+)\s*\S+/ ) {
                $FILER_C = $node->name();
                $count   = 1;
                my $disk = $1;
                push( @zns_drives, $disk );
                @zns_drives = unique(@zns_drives);

            }
        }
=cut

        my $prompts_answers1 = [ ".*to page down, .*" => ' ' ];
        my $output = $Api_Set_Obj->execute_raw_command( 'command' => "storage show disk -fields type", 'connectrec-match_table' => $prompts_answers1 );
        foreach my $line ( split( /\n/, $output ) ) {
            next if ( $line =~ /disk|type/ );
            if ( $line =~ /(\d+\.\d+\.\d+)\s+SSD-ZNS/ ) {
                $FILER_C = $node->name();
                $count   = 1;
                my $drive = $1;
                ( my $drv = $drive ) =~ s/\d+\.\d+\.//;
                $drv = "0n." . $drv;
                push( @zns_drives, $drv );
                @zns_drives = unique(@zns_drives);

            }
        }
        last if ( $count == 1 );
    }
    $NODE_A_CON    = $Nodes[0];
    $NODE_B_CON    = $Nodes[1];
    $FILER_A       = $Nodes[0]->name();
    $FILER_B       = $Nodes[1]->name();
    $Api_Set_Obj_A = $NODE_A_CON->get_7m_or_nodescope_apiset( connid => 'console' );
    $Api_Set_Obj_B = $NODE_B_CON->get_7m_or_nodescope_apiset( connid => 'console' );
    if ( $count == 1 ) {

        #my $assgn_flag=0;
        foreach my $drv (@zns_drives) {

            #my $Api_Set_Obj_C = $Api_Set_Obj;
            my $out = $Api_Set_Obj->execute_raw_command( 'command' => "run local disk assign $drv -o $FILER_C" );
            if ( $out =~ /(not accessible)|(disk does not exist)/ ) {
                logcomment("Disk $drv is not accessible on current node, trying on other node.");
                if ( $FILER_C eq $FILER_A ) {

                    #$Api_Set_Obj = $Api_Set_Obj_B;
                    my $out   = $Api_Set_Obj_B->execute_raw_command( 'command' => "run local disk assign $drv -o $FILER_B" );
                    my $unout = $Api_Set_Obj_B->execute_raw_command( 'command' => "run local disk show -n $drv" );
                    if ( $unout =~ /$drv\s+Not\s+Owned\s+/i ) {
                        logcomment("$drv drive is Not Owned, Assign drives to $FILER_B ");
                        $out   = $Api_Set_Obj_B->execute_raw_command( 'command' => "run local disk assign $drv -o $FILER_B" );
                        $unout = $Api_Set_Obj_B->execute_raw_command( 'command' => "run local disk show -n $drv" );
                        if ( $unout =~ /$drv\s+Not\s+Owned\s+/i ) {
                            logcomment("**FAIL** FAILED to assign drive to $FILER_B");
                            logresult("**FAIL** FAILED to assign drive to $FILER_B");

                        } else {
                            logcomment("Drive $drv has ownership assigned");
                        }
                    } else {
                        logcomment("Drive $drv has ownership assigned");
                    }
                    next;
                } else {

                    #$Api_Set_Obj = $Api_Set_Obj_A;
                    my $out   = $Api_Set_Obj_A->execute_raw_command( 'command' => "run local disk assign $drv -o $FILER_A" );
                    my $unout = $Api_Set_Obj_A->execute_raw_command( 'command' => "run local disk show -n $drv" );
                    if ( $unout =~ /$drv\s+Not\s+Owned\s+/i ) {
                        logcomment("$drv drive is Not Owned, Assign drives to $FILER_A");
                        $out   = $Api_Set_Obj_A->execute_raw_command( 'command' => "run local disk assign $drv -o $FILER_A" );
                        $unout = $Api_Set_Obj_A->execute_raw_command( 'command' => "run local disk show -n $drv" );
                        if ( $unout =~ /$drv\s+Not\s+Owned\s+/i ) {
                            logcomment("**FAIL** FAILED to assign drive to $FILER_A");
                            logresult("**FAIL** FAILED to assign drive to $FILER_A");

                        } else {
                            logcomment("Drive $drv has ownership assigned");
                        }
                    } else {
                        logcomment("Drive $drv has ownership assigned");
                    }
                    next;
                }

            }
            my $unout = $Api_Set_Obj->execute_raw_command( 'command' => "run local disk show -n $drv" );
            if ( $unout =~ /$drv\s+Not\s+Owned\s+/i ) {
                logcomment("$drv drive is Not Owned, Assign drives to $FILER_C ");
                $out   = $Api_Set_Obj->execute_raw_command( 'command' => "run local disk assign $drv -o $FILER_C" );
                $unout = $Api_Set_Obj->execute_raw_command( 'command' => "run local disk show -n $drv" );
                if ( $unout =~ /$drv\s+Not\s+Owned\s+/i ) {
                    logcomment("**FAIL** FAILED to assign drive to $FILER_C");
                    logresult("**FAIL** FAILED to assign drive to $FILER_C");

                } else {
                    logcomment("Drive $drv has ownership assigned");
                }
            } else {
                logcomment("Drive $drv has ownership assigned");
            }

            #$Api_Set_Obj = $Api_Set_Obj_C;

=head				
if($out =~ /Disk is in broken pool|Disk is currently owned by the partner/){
#logcomment("Disk assignment failed.Retrying again..")if($_==1);
logcomment("**FATAL** : Disk assign, remove ownership has NOT done properly");#if($_==2);
logresult( "FATAL", "DISK ASSIGN, REMOVE OWNERSHIP HAS NOT DONE PROPERLY" );#if($_==2);
#$assgn_flag=1;
#last;

}
=cut

            sleep(5);
        }

        #next if ($assgn_flag==1 && $_==1);
    } else {
        logcomment("**FATAL** : Cannot find ZNS drives on either of the Nodes.");
        logresult("**FATAL** : Cannot find ZNS drives on either of the Nodes.");
    }
    ( $count == 0 ) ? return 0 : return 1;

}

#########################################################
# Get owner where ZNS drive is found.
# Return Hash on Node where ZNS found.
#########################################################
sub get_zns_owner_node {
    my (%opts) = validate(
        @_,
        {
            node_present => { type => ARRAYREF },
            Handle       => { type => SCALAR, default => undef, optional => 1 },
        }
    );
    my @Nodes  = @{ $opts{node_present} };
    my $handle = $opts{Handle};
    my %nodes_filer;
    foreach my $node (@Nodes) {
        my $FILER_C = $node->name();
        my $FILER_A = $Nodes[0]->name();
        my $FILER_B = $Nodes[1]->name();
        logcomment("Node / filer : $FILER_C ");
        logcomment("Node A: $FILER_A | Node B :$FILER_B");
        logcomment("Check SDF Drive on filer $FILER_C");
        my $Transit_obj;
        my $timeout = 6000;
        $Transit_obj = NACL::Transit->new( name => $FILER_C );
        my $filer_state = $Transit_obj->get_state( 'timeout' => $timeout, 'get_state_timeout' => $timeout );
        logcomment("$FILER_C : Filer is in $filer_state ");
        my $zout;

        if ( $filer_state =~ /CLI|UP/ ) {
            my $Api_Set_Obj = $node->get_7m_or_nodescope_apiset( connid => 'console' );
            $Api_Set_Obj->execute_raw_command( 'command' => "\013" );
            $zout = $Api_Set_Obj->execute_raw_command( 'command' => "set test -confirmations off ;set -rows 0;disk show -fields type,owner" );
            foreach my $line ( split( /\n/, $zout ) ) {
                if ( $line =~ /ZNS/ ) {
                    if ( $line =~ /(\S+)\s+(\S+)\s+(\S+)/ ) {
                        logcomment("SDF Drive found - $1 with owner $2");
                        my $sdfown = $2;
                        my $sdfdrv = $1;
                        if ( $sdfown =~ /$FILER_A/ ) {
                            $nodes_filer{$FILER_A} = $Nodes[0];
                        } elsif ( $sdfown =~ /$FILER_B/ ) {
                            $nodes_filer{$FILER_B} = $Nodes[1];
                        }

                    }
                }
            }
        } elsif ( $filer_state =~ /MAINT/ ) {
            my $host       = host($FILER_C);
            my $API_Object = NACL::APISet->new(
                hostobj   => $host,
                category  => "Node",
                interface => "CLI",
                set       => "Maintenance"
            );
            $zout = $API_Object->execute_raw_command( 'command' => "disk_list" );
            if ( $zout =~ /ZNS/ ) {
                logcomment("ZNS/SDF drive found on Node $FILER_C , Test will be executing on $FILER_C Node");
                $nodes_filer{$FILER_C} = $node;
            }
        } else {
            logcomment("Filer is not in Maintenance or FileSystem mode, SDF drives cant be captured");
            return 1;
        }
    }
    return \%nodes_filer;
}
#########################################################
## ZNS identify commnads  - New subroutine after spec change
## Supports CLI
## Retruns Hash referene with all drv details
#########################################################
sub get_zns_identify_active_drv_details {
    my (%opts) = validate(
        @_,
        {
            Node_API => { type => OBJECT },
            Drives   => { type => ARRAYREF },
            Handle   => { type => SCALAR, default => undef, optional => 1 },
        }
    );
    my $Node_API = $opts{Node_API};
    my @drives   = @{ $opts{Drives} };
    my $handle   = $opts{Handle};
    logcomment("SDF : Identify commands for drive @drives ");
    my $pegnvme_drv_details = {};

    # New Standard SDF Identify Command

    foreach my $drive_id (@drives) {
        logcomment("Executing identify command for drive $drive_id");
        $pegnvme_drv_details->{$drive_id}->{'pegdrive'} = $drive_id;
        my ($id) = $drive_id =~ /peg_nvme(\d+)/;
        $pegnvme_drv_details->{$drive_id}->{'id'} = $id;
        my $out = $Node_API->execute_raw_command( 'command' => "sudo peg_nvmecontrol identify -c active_nsid_csi -s zoned $drive_id" );
        my ( $zone_nsp, $cns_nsp, $rd, $rds );
        foreach my $line ( split( /\n/, $out ) ) {
            if ( $line =~ /^IDs:/ ) {
                $rd = 1;
                next;
            }
            if ( $rd == 1 ) {
                $zone_nsp = $drive_id . "ns1" if ( $line == 1 );
                $zone_nsp = $drive_id . "ns2" if ( $line == 2 );
            }
        }
        my $out = $Node_API->execute_raw_command( 'command' => "sudo peg_nvmecontrol identify -c active_nsid_csi -s nvm $drive_id" );
        foreach my $line ( split( /\n/, $out ) ) {
            if ( $line =~ /^IDs:/ ) {
                $rds = 1;
                next;
            }
            if ( $rds == 1 ) {
                $cns_nsp = $drive_id . "ns1" if ( $line == 1 );
                $cns_nsp = $drive_id . "ns2" if ( $line == 2 );
            }
        }
        logcomment("Zone NSP : $zone_nsp \n");
        logcomment("CNS NSP : $cns_nsp \n");
        $pegnvme_drv_details->{$drive_id}->{'cns_nsp'}  = $cns_nsp;
        $pegnvme_drv_details->{$drive_id}->{'zone_nsp'} = $zone_nsp;
        my $output = $Node_API->execute_raw_command( 'command' => "sudo peg_nvmecontrol identify $drive_id" );
        my ( $head, $key, $value );
        foreach my $line ( split( /\n/, $output ) ) {
            next if ( $line =~ /^==/ );
            if ( $line !~ /(.*):/ ) {
                $head = $line;
            }
            if ( ( $line =~ /(.*):\s+(.*)/ ) ) {
                my $key   = $1;
                my $value = $2;
                $key   =~ s/\s+/_/g;
                $value =~ s/\s+/_/g;
                $head  =~ s/\s+/_/g;
                $key = $head . $key if ( $key =~ /^\_/ );
                $pegnvme_drv_details->{$drive_id}->{$key} = $value;
            }
        }
        my $zonenspdrv = $pegnvme_drv_details->{$drive_id}->{'zone_nsp'};
        my $cnspdrv    = $pegnvme_drv_details->{$drive_id}->{'cns_nsp'};
        logcomment("Zone NSP Drive : $zonenspdrv");
        my $out = $Node_API->execute_raw_command( 'command' => "sudo peg_nvmecontrol identify $zonenspdrv" );
        my ( $head, $key, $value, $meta );
        foreach my $line ( split( /\n/, $out ) ) {
            next if ( $line =~ /^==/ );
            if ( $line !~ /(.*):/ ) {
                $head = $line;
            }
            if ( $line =~ /#/ ) {
                if ( $line =~ /(.*):\s+Data\s+Size:(.*)\s+Metadata Size:(.*)/ ) {
                    my $key  = $1;
                    my $data = $2;
                    my $meta = $3;
                    $key  =~ s/\s+/_/g;
                    $data =~ s/\s+//g;
                    $meta =~ s/\s+//g;
                    $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{ $key . "_Data Size" }     = $data;
                    $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{ $key . "_Metadata Size" } = $meta;
                }
            }
            if ( ( $line =~ /(.*):\s+(.*)/ ) ) {
                next if ( $line =~ /\#/ );
                my $key   = $1;
                my $value = $2;
                $key   =~ s/\s+/_/g;
                $value =~ s/\s+/_/g;
                $head  =~ s/\s+/_/g;
                $key = $head . $key if ( $key =~ /^\_/ );
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{$key} = $value;
            }
        }
        my $out = $Node_API->execute_raw_command( 'command' => "sudo peg_nvmecontrol identify $cnspdrv" );
        my ( $head, $key, $value, $meta );
        foreach my $line ( split( /\n/, $out ) ) {
            next if ( $line =~ /^==/ );
            if ( $line !~ /(.*):/ ) {
                $head = $line;
            }
            if ( $line =~ /#/ ) {
                if ( $line =~ /(.*):\s+Data\s+Size:(.*)\s+Metadata Size:(.*)/ ) {
                    my $key  = $1;
                    my $data = $2;
                    my $meta = $3;
                    $key  =~ s/\s+/_/g;
                    $data =~ s/\s+//g;
                    $meta =~ s/\s+//g;
                    $pegnvme_drv_details->{$drive_id}->{$cnspdrv}->{ $key . "_Data Size" }     = $data;
                    $pegnvme_drv_details->{$drive_id}->{$cnspdrv}->{ $key . "_Metadata Size" } = $meta;
                }
            }
            if ( ( $line =~ /(.*):\s+(.*)/ ) ) {
                next if ( $line =~ /\#/ );
                my $key   = $1;
                my $value = $2;
                $key   =~ s/\s+/_/g;
                $value =~ s/\s+/_/g;
                $head  =~ s/\s+/_/g;
                $key = $head . $key if ( $key =~ /^\_/ );
                $pegnvme_drv_details->{$drive_id}->{$cnspdrv}->{$key} = $value;
            }
        }

        my $out = $Node_API->execute_raw_command( 'command' => "sudo peg_nvmecontrol identify -c ns_csi -s zoned $zonenspdrv" );
        my ( $head, $key, $value, $meta );
        foreach my $line ( split( /\n/, $out ) ) {
            next if ( $line =~ /^==/ );
            if ( $line !~ /(.*):/ ) {
                $head = $line;
            }
            if ( ( $line =~ /(.*):\s+(.*)/ ) ) {
                my $key   = $1;
                my $value = $2;
                $key   =~ s/\s+/_/g;
                $value =~ s/\s+/_/g;
                $head  =~ s/\s+/_/g;
                $key = $head . $key if ( $key =~ /^\_/ );
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{$key} = $value;
            }
        }

        my $out = $Node_API->execute_raw_command( 'command' => "sudo peg_nvmecontrol identify -c ctrlr_csi -s zoned $drive_id" );
        my ( $head, $key, $value, $meta );
        foreach my $line ( split( /\n/, $out ) ) {
            next if ( $line =~ /^==/ );
            if ( $line !~ /(.*):/ ) {
                $head = $line;
            }
            if ( ( $line =~ /(.*):\s+(.*)/ ) ) {
                my $key   = $1;
                my $value = $2;
                $key   =~ s/\s+/_/g;
                $value =~ s/\s+/_/g;
                $head  =~ s/\s+/_/g;
                $key = $head . $key if ( $key =~ /^\_/ );
                $pegnvme_drv_details->{$drive_id}->{$key} = $value;
            }
        }
        my $local_vendor = $pegnvme_drv_details->{$drive_id}->{'Vendor_ID'};
        logcomment("Drive Vendor ID : $local_vendor");
        if ( $local_vendor =~ /144d/ ) {
            $pegnvme_drv_details->{$drive_id}->{'zns_vendor_name'} = "samsung";
        } else {
            $pegnvme_drv_details->{$drive_id}->{'zns_vendor_name'} = "wd";
        }
    }    #End of foreach drive
         #logcomment( "Data" . Dumper($pegnvme_drv_details) );
    return $pegnvme_drv_details;
}    #End of Main sub

sub detect_zns_drives {
    my (%opts) = validate( @_, { node => { type => OBJECT }, } );
    my $Node = $opts{node};
    my @zns_drives       = ();
    my $Api_Set_Obj      = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
    my $prompts_answers1 = [ ".*to page down, .*" => ' ' ];
    my $output           = $Api_Set_Obj->execute_raw_command( 'command' => "storage show disk -fields type", 'connectrec-match_table' => $prompts_answers1 );
    foreach my $line ( split( /\n/, $output ) ) {
        next if ( $line =~ /disk|type/ );
        if ( $line =~ /(\d+\.\d+\.\d+)\s+SSD-ZNS/ ) {
            my $drive = $1;
            ( my $drv = $drive ) =~ s/\d+\.\d+\.//;
            $drv = "0n." . $drv;
            push( @zns_drives, $drv );
            @zns_drives = unique(@zns_drives);

        }
    }
    return @zns_drives;
}
#########################################################
### Get drive details - zns and systemshell drives
### Supports - Only CLI / file system
### Returns  - Array of systemshell drives and zns drives.
##########################################################
sub get_zns_systemshell_drvs {
    my (%opts) = validate(
        @_,
        {
            Node   => { type => OBJECT },
            handle => { type => SCALAR, optional => 1, default => '' },
        }
    );
    my $Node       = $opts{Node};
    my $drv_result = {};
    my $Api_Set_Obj;
    my $Transit_obj;
    my $FILER_C     = $Node->name();
    my $timeout     = 6000;
    my $host        = host($FILER_C);
    my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
    $Api_Set_Obj->set_timeout( "connectrec-timeout" => $timeout );
    my $ss_disk_list = $Api_Set_Obj->execute_raw_command( 'command' => "set -rows 0;systemshel -node $FILER_C -command \"sudo peg_nvmecontrol devlist\"" );
    logcomment("Getting drive details from systemshell");

    foreach my $line ( split( /\n/, $ss_disk_list ) ) {
        my ( $drv, $ns, $nsz, $npid );
        if ( $line =~ /(\S+)\:\s+(\S+)/ ) {
            $drv = $1;
            $drv_result->{$drv}->{'PRDID'} = $2;
        }
        if ( $line =~ /(\S+)ns(\d+)/ ) {
            $drv  = $1;
            $npid = $2;
            if ( $line =~ /(\S+)\s+\((\S+)\)/ ) {
                $ns  = $1;
                $nsz = $2;
                if ( $nsz != /^0/ ) {
                    $drv_result->{$drv}->{$npid}->{'NAME_SP'} = $ns;
                    $drv_result->{$drv}->{$npid}->{'NS_SIZE'} = $nsz;
                    $drv_result->{$drv}->{$npid}->{'NS_ID'}   = $npid;
                }
            }
        }
    }
    my @nv_drv = keys(%$drv_result);
    logcomment("System Shell Drive(s) : @nv_drv");
    my @zns_drvs;
    foreach my $dr ( sort keys %$drv_result ) {
        my $id_count = 0;
        if ( $drv_result->{$dr}->{'1'} ) {
            $id_count++;
        }
        if ( $drv_result->{$dr}->{'2'} ) {
            $id_count++;
        }
        if ( $id_count == 2 ) {
            logcomment("ZNS drive : $dr");
            push( @zns_drvs, $dr );
        }
    }
    return ( \@nv_drv, \@zns_drvs );
}
#########################################################
## Get drive details - disk_list and stoarage disk
## Supports - Only CLI / file system
#########################################################
sub zns_get_disk_info {
    my (%opts) = validate(
        @_,
        {
            Node   => { type => OBJECT },
            handle => { type => SCALAR, optional => 1, default => '' },
        }
    );
    my $Node                  = $opts{Node};
    my $zns_ontap_drv_details = {};

    logcomment("Getting drive details from disk_list and storage disk command");
    my $Api_Set_Obj;
    my $Transit_obj;
    my $FILER_C = $Node->name();
    my $timeout = 6000;
    $Transit_obj = NACL::Transit->new( name => $FILER_C );
    my $filer_state = $Transit_obj->get_state( 'timeout' => 3600,     'get_state_timeout' => 7200 );
    my $filer_state = $Transit_obj->get_state( 'timeout' => $timeout, 'get_state_timeout' => $timeout );
    logcomment("$FILER_C : Filer is in $filer_state ");

    if ( $filer_state =~ /CLI|UP/ ) {
        my $host = host($FILER_C);
        my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
        $Api_Set_Obj->set_timeout( "connectrec-timeout" => $timeout );
        my $out = $Api_Set_Obj->execute_raw_command( 'command' => "set -rows 0;run local disk_list" );
        foreach my $line ( split( /\n/, $out ) ) {
            next if ( $line =~ /DISK/ );
            next if ( $line =~ /^--------/ );
            if ( $line =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/ ) {
                my $disk      = $1;
                my $vendor    = $2;
                my $prod_id   = $3;
                my $firmware  = $4;
                my $serial_no = $5;
                my $hw        = $6;
                my $blks      = $7;
                my $bps       = $8;
                my $dq        = $9;
                $zns_ontap_drv_details->{$serial_no};
                $zns_ontap_drv_details->{$serial_no}->{'disk'}      = $disk;
                $zns_ontap_drv_details->{$serial_no}->{'vendor'}    = $vendor;
                $zns_ontap_drv_details->{$serial_no}->{'prod_id'}   = $prod_id;
                $zns_ontap_drv_details->{$serial_no}->{'firmware'}  = $firmware;
                $zns_ontap_drv_details->{$serial_no}->{'serial_no'} = $serial_no;
                $zns_ontap_drv_details->{$serial_no}->{'hw'}        = $hw;
                $zns_ontap_drv_details->{$serial_no}->{'blks'}      = $blks;
                $zns_ontap_drv_details->{$serial_no}->{'bps'}       = $bps;
                $zns_ontap_drv_details->{$serial_no}->{'dq'}        = $dq;
            }
        }
        my $out = $Api_Set_Obj->execute_raw_command( 'command' => "set -rows 0;storage disk show -field serial-number,usable-size,owner,container-type" );
        foreach my $line ( split( /\n/, $out ) ) {
            next if ( $line =~ /disk/ );
            next if ( $line =~ /^--------/ );
            if ( $line =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/ ) {
                my $cm_disk        = $1;
                my $owner          = $2;
                my $container_type = $3;
                my $serialno       = $4;
                my $use_size       = $5;
                $zns_ontap_drv_details->{$serialno}->{'c_disk'}         = $cm_disk;
                $zns_ontap_drv_details->{$serialno}->{'owner'}          = $owner;
                $zns_ontap_drv_details->{$serialno}->{'container_type'} = $container_type;
                $zns_ontap_drv_details->{$serialno}->{'usable_size'}    = $container_type;
            }
        }
    } else {
        logcomment("Filer must be in CLI ");

        #need to implement maintenance mode drive details.
    }
    return $zns_ontap_drv_details;
}

sub zns_pre_test {
    my (%opts) = validate(
        @_,
        {
            filer_mode      => { type => SCALAR },
            node_present    => { type => ARRAYREF },
            Test            => { type => OBJECT },
            change_state_to => { type => SCALAR, optional => 1, default => 'CLI' },
            LOGDIR          => { type => SCALAR, optional => 1, default => '' },
            test_setup      => { type => SCALAR, optional => 1, default => 'SAS' },
            BOOT_TYPE       => { type => SCALAR, optional => 1, default => 'A' },
            Tahiti          => { type => SCALAR, optional => 1, default => 'no' },
        }
    );
    my @Nodes;

    #handle_run_local_error();
    if ( defined( @{ $opts{node_present} } ) ) {
        @Nodes = @{ $opts{node_present} };
    } else {
        logcomment("Finding Nodes ......");
        @Nodes = NACL::C::Node->find();
    }
    Storage::Common_Lib::handle_run_local_error( Nodes => [@Nodes] );
    my $Tahiti          = $opts{Tahiti};
    my $Filer_mode      = $opts{filer_mode};
    my $Test            = $opts{Test};
    my $change_to       = $opts{change_state_to};
    my $TEST_CONFIG     = $opts{test_config};
    my $test_setup      = $opts{test_setup};
    my $exit_test       = 0;
    my @Filer_array     = ();
    my $execution_state = $change_to;
    my @disk_list;
    my $node_ref;
    my @disk_array;
    my %FILER_API = ();
    my @disk_array;

    # my $timeout;
    my $Api_Set_Obj;
    my $disk_capacity_total = 0;
    my $LOGDIR              = $opts{LOGDIR};
    my $params              = NATE::ParamSet->new( global => 1 );
    my $EOE                 = $params->get( 'EOE', default => 'default' );
    my $EMAIL               = $params->get( 'EMAIL', default => 'y' );
    my $MAIL_TO             = $params->get( 'MAIL_TO', default => 'Email to' );
    my $MAIL_FROM           = $params->get( 'MAIL_FROM', default => 'Email from' );

    if ( $opts{LOGDIR} ) {
        $LOGDIR = $opts{LOGDIR};
    } else {
        $LOGDIR = $ENV{HOME};
    }
    logcomment("Test Setup selected : $test_setup");
    my $TEST_NAME;
    opendir( DIR_LOG31, $LOGDIR );
    my @files_DIR_LOG31 = readdir(DIR_LOG31);
    foreach my $file (@files_DIR_LOG31) {
        if ( ( $file =~ /(.*_NADQ02.*)\.log/ ) && ( $file !~ /results|END_FILER_CONFIG/ ) ) {
            $TEST_NAME = $1;
        }
    }
    $node_ref = \@Nodes;
    foreach my $temp_node (@Nodes) {
        my $Filer = $temp_node->name();
        push( @Filer_array, $Filer );
    }

    # Check state of filer and Execute based on state
    logcomment("Nodes present - @Filer_array");
    logcomment("Log loction - $LOGDIR");
    logcomment("Apollo-Tahiti : $Tahiti");
    my @chk_state = ();
    foreach my $Node (@Nodes) {
        my $Transit_obj;
        my $FILER_C = $Node->name();
        $Transit_obj = NACL::Transit->new( name => $FILER_C );
        my $filer_state = $Transit_obj->get_state( 'timeout' => $timeout, 'get_state_timeout' => $timeout );
        $Test->nlog("$FILER_C : Filer is in $filer_state ");
        push( @chk_state, $filer_state );
    }
    my $len = @chk_state;
    if ( $len > 1 ) {
        if ( $chk_state[0] eq $chk_state[1] ) {
            $execution_state = $chk_state[0];
            $Test->nlog("Execution state - $execution_state\n");
        } else {
            $Test->nlog("States of filer is different, Boot filer in same mode");
        }
    } else {

        #$execution_state = $chk_state[0];
        $Test->nlog("Execution state - $execution_state\n");
    }

    if ( $execution_state eq 'MAINT' ) {
        foreach my $Node (@Nodes) {
            my $Filer      = $Node->name();
            my $host       = host($Filer);
            my $API_Object = NACL::APISet->new(
                hostobj   => $host,
                category  => "Node",
                interface => "CLI",
                set       => "Maintenance"
            );
            $FILER_API{$Filer} = $API_Object;
        }
    }

    # Check filer State and change to required state
    $Test->nlog("##################################################################################");
    $Test->nlog("##");
    $Test->nlog("## Begining of PRE_TEST logs for Filer : @Filer_array");
    $Test->nlog("##");
    $Test->nlog("##################################################################################");

    foreach my $Node (@Nodes) {
        $Test->nlog("===================================================================================");
        $Test->nlog("==== PRE_TEST STEP 1 : Get pegnvme drive details");
        $Test->nlog("===================================================================================");

        my $FILER_C = $Node->name();
        my $Host    = host($FILER_C);
        my $Api_Set_Obj_SS;
        my $drv_result = {};
        if ( $execution_state eq 'CLI' ) {
            my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
            logcomment("Check Current Privilege Level ");
            $Api_Set_Obj->execute_raw_command( 'command' => "set" );
            logcomment("Set privilege level to diag");
            my $prompts_answers = [ ".*Do you want to continue.*" => 'y' ];
            $Api_Set_Obj->execute_raw_command( 'command' => "set diag", 'connectrec-match_table' => $prompts_answers );
            logcomment("Privilege level diag set");
            $Api_Set_Obj->execute_raw_command( 'command' => "set" );
            logcomment("Get password to login in system shell");
            my $Passwd = $Host->default_password();
            logcomment("Filer password : $Passwd");
            my $prompts_answers = [ ".*Do you want to continue.*" => 'y' ];
            $Api_Set_Obj_SS = NACL::APISet->new(
                hostobj   => $Host,
                category  => "Node",
                interface => "CLI",
                set       => "Systemshell"
            );
        } else {
            my $API_Object = NACL::APISet->new(
                hostobj   => $Host,
                category  => "Node",
                interface => "CLI",
                set       => "Maintenance"
            );
            my $Passwd = $Host->default_password();
            my $prompts_answers = [ ".*password*" => "$Passwd" ];
            $API_Object->execute_raw_command( 'command' => "systemshell", 'connectrec-match_table' => $prompts_answers );
            $API_Object->execute_raw_command( 'command' => "\013" );
            $Api_Set_Obj_SS = $API_Object;
        }
        logcomment("System shell API created, getting pegnvme drives ");
        my @nv_drv;
        my $get_drvs_file = 0;
        if ( $get_drvs_file == 1 ) {
            logcomment("Getting drive from file");
            my $nv_drvf = get_pegnvme_from_file();
            logcomment("Drives are : @$nv_drvf");
            @nv_drv = @$nv_drvf;
        } else {
            my ( $sys_shell_drvs, $zns_drvs ) = get_zns_systemshell_drvs( Node => $Node );
            logcomment("System Shell Drive(s) : @$sys_shell_drvs");
            logcomment("SDF /ZNS Drive(s) : @$zns_drvs");
            @nv_drv = @$zns_drvs;
        }
        logcomment("Drives identified are : @nv_drv");
        $Test->nlog("===================================================================================");
        $Test->nlog("==== PRE_TEST STEP 2 : Display identify commands details for all drives");
        $Test->nlog("===================================================================================");

        #my $peg_drv_det = zns_identify_cmds( Node_API => $Api_Set_Obj_SS, Drives => \@nv_drv );
        my $peg_drv_det = get_zns_identify_active_drv_details( Node_API => $Api_Set_Obj_SS, Drives => \@nv_drv );

        #logcomment("Display identify commands details for all drives");
        #disp_pegnvme_drive_variables( peg_nvme_hash_details => $peg_drv_det, Test => $Test );

        $Test->nlog("===================================================================================");
        $Test->nlog("==== PRE_TEST STEP 3 : Display Log Pages details ");
        $Test->nlog("===================================================================================");
        logcomment("Display log pages for all drives");
        logcomment("Get log pages for all SDF drives");
        logcomment("Drives are : @nv_drv");
        foreach my $ddr (@nv_drv) {
            logcomment("Get Log page 1 for drive : $ddr");
            $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol logpage -p 1 $ddr" );
            $Api_Set_Obj_SS->execute_raw_command( 'command' => "\013" );
            logcomment("Get Log Page 2 for drive : $ddr");
            $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol logpage -p 2 $ddr" );
            logcomment("Get Log Page C0 for drive : $ddr");
            $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol logpage -p 0xC0 $ddr" );
        }
        logcomment("Log page details - End ");

        $Test->nlog("===================================================================================");
        $Test->nlog("==== PRE_TEST STEP 4 : Run SDF Zone management commands ");
        $Test->nlog("===================================================================================");
        foreach my $ddr (@nv_drv) {
            my $zns_drive = $peg_drv_det->{$ddr}->{'zone_nsp'};
            if ( $zns_drive !~ /^$/ ) {
                logcomment("Run SDF Zone management commands on drive: $zns_drive");
                $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol zone_mgmt receive -c -l 0 -n 10 -z Closed $zns_drive" );
                $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol zone_mgmt receive -c -l 0 -n 10 -z Empty $zns_drive" );
                $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol zone_mgmt receive -c -l 0 -n 10 -z ExpOpen $zns_drive" );
                $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol zone_mgmt receive -c -l 0 -n 10 -z ImpOpen $zns_drive" );
                $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol zone_mgmt receive -c -l 0 -n 10 -z Full $zns_drive" );
                $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol zone_mgmt receive -c -l 0 -n 10 -z Offline $zns_drive" );
                $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol zone_mgmt receive -c -l 0 -n 10 -z RdOnly $zns_drive" );
            }
        }
        $Test->nlog("===================================================================================");
        $Test->nlog("==== PRE_TEST STEP 5 : Display SDF drive using disk_list ");
        $Test->nlog("===================================================================================");
        my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
        $Api_Set_Obj->execute_raw_command( 'command' => "set -rows -0;priv set test;run local disk_list" );

        $Test->nlog("===================================================================================");
        $Test->nlog("==== PRE_TEST STEP 6 : Display System details ");
        $Test->nlog("===================================================================================");
        my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
        $Api_Set_Obj->execute_raw_command( 'command' => "set -rows -0;priv set test;run local sysconfig -v" );

        $Test->nlog("===================================================================================");
        $Test->nlog("==== PRE_TEST STEP 7 : Show Link Status ");
        $Test->nlog("===================================================================================");
        my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
        $Api_Set_Obj->execute_raw_command( 'command' => "set -rows -0;priv set test;storage show disk -stat -disk *" );

        $Test->nlog("===================================================================================");
        $Test->nlog("==== PRE_TEST STEP 8 : Show filer Mode Disk Stats ");
        $Test->nlog("===================================================================================");
        my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
        $Api_Set_Obj->execute_raw_command( 'command' => "set -rows -0;priv set test;run local storage show" );
        $Test->nlog("===================================================================================");
        $Test->nlog("==== PRE_TEST STEP 9  : Check for drive path on both nodes ");
        $Test->nlog("===================================================================================");

        foreach my $Node (@Nodes) {
            my $FILER_C = $Node->name();
            if ( $execution_state eq 'CLI' ) {
                my $Host = host($FILER_C);
                my $Api_Set_Obj_SS;
                my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
                $Api_Set_Obj->execute_raw_command( 'command' => "\013" );
                $Api_Set_Obj->execute_raw_command( 'command' => "\013" );
                my @temp_nodes = ();
                push( @temp_nodes, $Node );
                my $arr_ref_temp = \@temp_nodes;
                my $outpt_chk_drv_path;
                $outpt_chk_drv_path = zns_check_drv_path( node_present => $arr_ref_temp );

                if ( $outpt_chk_drv_path == 1 ) {
                    logresult( "FATAL", "Drive path missing" );
                }
            } else {
                logcomment("FILER: $FILER_C IS IN MAINTENANCE MODE, COMMAND \"storage disk show\" IS NOT SUPPORTED");
            }
        }

    }

    $Test->nlog("##################################################################################");
    $Test->nlog("##");
    $Test->nlog("## End of PRE_TEST logs for Filer : @Filer_array");
    $Test->nlog("##");
    $Test->nlog("##################################################################################");

}

sub zns_post_test {
    my (%opts) = validate(
        @_,
        {
            filer_mode      => { type => SCALAR },
            node_present    => { type => ARRAYREF },
            Test            => { type => OBJECT },
            change_state_to => { type => SCALAR, optional => 1, default => 'CLI' },
            LOGDIR          => { type => SCALAR, optional => 1, default => '' },
            test_setup      => { type => SCALAR, optional => 1, default => 'SAS' },
            BOOT_TYPE       => { type => SCALAR, optional => 1, default => 'A' },
            Tahiti          => { type => SCALAR, optional => 1, default => 'no' },
        }
    );
    my @Nodes;

    if ( defined( @{ $opts{node_present} } ) ) {
        @Nodes = @{ $opts{node_present} };
    } else {
        logcomment("Finding Nodes ......");
        @Nodes = NACL::C::Node->find();
    }
    Storage::Common_Lib::handle_run_local_error( Nodes => [@Nodes] );
    my $Tahiti          = $opts{Tahiti};
    my $Filer_mode      = $opts{filer_mode};
    my $Test            = $opts{Test};
    my $change_to       = $opts{change_state_to};
    my $TEST_CONFIG     = $opts{test_config};
    my $test_setup      = $opts{test_setup};
    my $exit_test       = 0;
    my @Filer_array     = ();
    my $execution_state = $change_to;
    my @disk_list;
    my $node_ref;
    my @disk_array;
    my %FILER_API = ();
    my @disk_array;
    my $Api_Set_Obj;
    my $disk_capacity_total = 0;
    my $LOGDIR              = $opts{LOGDIR};
    my $params              = NATE::ParamSet->new( global => 1 );
    my $EOE                 = $params->get( 'EOE', default => 'default' );
    my $EMAIL               = $params->get( 'EMAIL', default => 'y' );
    my $MAIL_TO             = $params->get( 'MAIL_TO', default => 'Email to' );
    my $MAIL_FROM           = $params->get( 'MAIL_FROM', default => 'Email from' );

    if ( $opts{LOGDIR} ) {
        $LOGDIR = $opts{LOGDIR};
    } else {
        $LOGDIR = $ENV{HOME};
    }
    logcomment("Test Setup selected : $test_setup");
    my $TEST_NAME;
    opendir( DIR_LOG31, $LOGDIR );
    my @files_DIR_LOG31 = readdir(DIR_LOG31);
    foreach my $file (@files_DIR_LOG31) {
        if ( ( $file =~ /(.*_NADQ02.*)\.log/ ) && ( $file !~ /results|END_FILER_CONFIG/ ) ) {
            $TEST_NAME = $1;
        }
    }
    $node_ref = \@Nodes;
    foreach my $temp_node (@Nodes) {
        my $Filer = $temp_node->name();
        push( @Filer_array, $Filer );
    }

    logcomment("Nodes present - @Filer_array");
    logcomment("Log loction - $LOGDIR");
    logcomment("Apollo-Tahiti : $Tahiti");
    my @chk_state = ();
    foreach my $Node (@Nodes) {
        my $Transit_obj;
        my $FILER_C = $Node->name();
        $Transit_obj = NACL::Transit->new( name => $FILER_C );
        my $filer_state = $Transit_obj->get_state( 'timeout' => $timeout, 'get_state_timeout' => $timeout );
        $Test->nlog("$FILER_C : Filer is in $filer_state ");
        push( @chk_state, $filer_state );
    }
    my $len = @chk_state;
    if ( $len > 1 ) {
        if ( $chk_state[0] eq $chk_state[1] ) {
            $execution_state = $chk_state[0];
            $Test->nlog("Execution state - $execution_state\n");
        } else {
            $Test->nlog("States of filer is different, Boot filer in same mode");
        }
    } else {

        $execution_state = $chk_state[0];
        $Test->nlog("Execution state - $execution_state\n");
    }

    if ( $execution_state eq 'MAINT' ) {
        foreach my $Node (@Nodes) {
            my $Filer      = $Node->name();
            my $host       = host($Filer);
            my $API_Object = NACL::APISet->new(
                hostobj   => $host,
                category  => "Node",
                interface => "CLI",
                set       => "Maintenance"
            );
            $FILER_API{$Filer} = $API_Object;
        }
    }

    $Test->nlog("##################################################################################");
    $Test->nlog("##");
    $Test->nlog("## Begining of POST_TEST logs for Filer : @Filer_array");
    $Test->nlog("##");
    $Test->nlog("##################################################################################");

    foreach my $Node (@Nodes) {
        $Test->nlog("===================================================================================");
        $Test->nlog("==== POST_TEST STEP 1 : Get pegnvme drive details");
        $Test->nlog("===================================================================================");

        my $FILER_C = $Node->name();
        my $Host    = host($FILER_C);
        my $Api_Set_Obj_SS;
        my $drv_result = {};
        if ( $execution_state eq 'CLI' ) {
            my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
            logcomment("Check Current Privilege Level ");
            $Api_Set_Obj->execute_raw_command( 'command' => "set" );
            logcomment("Set privilege level to diag");
            my $prompts_answers = [ ".*Do you want to continue.*" => 'y' ];
            $Api_Set_Obj->execute_raw_command( 'command' => "set diag", 'connectrec-match_table' => $prompts_answers );
            logcomment("Privilege level diag set");
            $Api_Set_Obj->execute_raw_command( 'command' => "set" );
            logcomment("Get password to login in system shell");
            my $Passwd = $Host->default_password();
            logcomment("Filer password : $Passwd");
            my $prompts_answers = [ ".*Do you want to continue.*" => 'y' ];
            $Api_Set_Obj_SS = NACL::APISet->new(
                hostobj   => $Host,
                category  => "Node",
                interface => "CLI",
                set       => "Systemshell"
            );
        } else {
            my $API_Object = NACL::APISet->new(
                hostobj   => $Host,
                category  => "Node",
                interface => "CLI",
                set       => "Maintenance"
            );
            my $Passwd = $Host->default_password();
            my $prompts_answers = [ ".*password*" => "$Passwd" ];
            $API_Object->execute_raw_command( 'command' => "systemshell", 'connectrec-match_table' => $prompts_answers );
            $API_Object->execute_raw_command( 'command' => "\013" );
            $Api_Set_Obj_SS = $API_Object;
        }
        logcomment("System shell API created, getting pegnvme drives ");
        my @nv_drv;
        my $get_drvs_file = 0;
        if ( $get_drvs_file == 1 ) {
            logcomment("Getting drive from file");
            my $nv_drvf = get_pegnvme_from_file();
            logcomment("Drives are : @$nv_drvf");
            @nv_drv = @$nv_drvf;
        } else {
            my ( $sys_shell_drvs, $zns_drvs ) = get_zns_systemshell_drvs( Node => $Node );
            logcomment("System Shell Drive(s) : @$sys_shell_drvs");
            logcomment("SDF /ZNS Drive(s) : @$zns_drvs");
            @nv_drv = @$zns_drvs;
        }
        logcomment("Drives identified are : @nv_drv");
        $Test->nlog("===================================================================================");
        $Test->nlog("==== POST_TEST STEP 2 : Display identify commands details for all drives");
        $Test->nlog("===================================================================================");

        my $peg_drv_det = get_zns_identify_active_drv_details( Node_API => $Api_Set_Obj_SS, Drives => \@nv_drv );

        #logcomment("Display identify commands details for all drives");
        #disp_pegnvme_drive_variables( peg_nvme_hash_details => $peg_drv_det, Test => $Test );

        $Test->nlog("===================================================================================");
        $Test->nlog("==== POST_TEST STEP 3 : Display Log Pages details ");
        $Test->nlog("===================================================================================");
        logcomment("Display log pages for all drives");
        logcomment("Get log pages for all SDF drives");
        logcomment("Drives are : @nv_drv");
        foreach my $ddr (@nv_drv) {
            logcomment("Get Log page 1 for drive : $ddr");
            $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol logpage -p 1 $ddr" );
            $Api_Set_Obj_SS->execute_raw_command( 'command' => "\013" );
            logcomment("Get Log Page 2 for drive : $ddr");
            $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol logpage -p 2 $ddr" );
            logcomment("Get Log Page C0 for drive : $ddr");
            $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol logpage -p 0xC0 $ddr" );
        }
        logcomment("Log page details - End ");
        $Test->nlog("===================================================================================");
        $Test->nlog("==== POST_TEST STEP 4 : Run SDF Zone management commands ");
        $Test->nlog("===================================================================================");
        foreach my $ddr (@nv_drv) {
            my $zns_drive = $peg_drv_det->{$ddr}->{'zone_nsp'};
            if ( $zns_drive !~ /^$/ ) {
                logcomment("Run SDF Zone management commands on drive: $zns_drive");
                $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol zone_mgmt receive -c -l 0 -n 10 -z Closed $zns_drive" );
                $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol zone_mgmt receive -c -l 0 -n 10 -z Empty $zns_drive" );
                $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol zone_mgmt receive -c -l 0 -n 10 -z ExpOpen $zns_drive" );
                $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol zone_mgmt receive -c -l 0 -n 10 -z ImpOpen $zns_drive" );
                $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol zone_mgmt receive -c -l 0 -n 10 -z Full $zns_drive" );
                $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol zone_mgmt receive -c -l 0 -n 10 -z Offline $zns_drive" );
                $Api_Set_Obj_SS->execute_raw_command( 'command' => "sudo peg_nvmecontrol zone_mgmt receive -c -l 0 -n 10 -z RdOnly $zns_drive" );
            }
        }
        $Test->nlog("===================================================================================");
        $Test->nlog("==== POST_TEST STEP 5 : Display SDF drive using disk_list ");
        $Test->nlog("===================================================================================");
        my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
        $Api_Set_Obj->execute_raw_command( 'command' => "set -rows -0;priv set test;run local disk_list" );

        $Test->nlog("===================================================================================");
        $Test->nlog("==== POST_TEST STEP 6 : Display System details ");
        $Test->nlog("===================================================================================");
        my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
        $Api_Set_Obj->execute_raw_command( 'command' => "set -rows -0;priv set test;run local sysconfig -v" );

        $Test->nlog("===================================================================================");
        $Test->nlog("==== POST_TEST STEP 7 : Show Link Status ");
        $Test->nlog("===================================================================================");
        my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
        $Api_Set_Obj->execute_raw_command( 'command' => "set -rows -0;priv set test;storage show disk -stat -disk *" );

        $Test->nlog("===================================================================================");
        $Test->nlog("==== POST_TEST STEP 8 : Show filer Mode Disk Stats ");
        $Test->nlog("===================================================================================");
        my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
        $Api_Set_Obj->execute_raw_command( 'command' => "set -rows -0;priv set test;run local storage show" );
        $Test->nlog("===================================================================================");
        $Test->nlog("==== POST_TEST STEP 9  : Check for drive path on both nodes ");
        $Test->nlog("===================================================================================");

        foreach my $Node (@Nodes) {
            my $FILER_C = $Node->name();
            if ( $execution_state eq 'CLI' ) {
                my $Host = host($FILER_C);
                my $Api_Set_Obj_SS;
                my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset( connid => 'console' );
                $Api_Set_Obj->execute_raw_command( 'command' => "\013" );
                $Api_Set_Obj->execute_raw_command( 'command' => "\013" );
                my @temp_nodes = ();
                push( @temp_nodes, $Node );
                my $arr_ref_temp = \@temp_nodes;
                my $outpt_chk_drv_path;
                $outpt_chk_drv_path = zns_check_drv_path( node_present => $arr_ref_temp );

                if ( $outpt_chk_drv_path == 1 ) {
                    logresult( "FATAL", "Drive path missing" );
                }
            } else {
                logcomment("FILER: $FILER_C IS IN MAINTENANCE MODE, COMMAND \"storage disk show\" IS NOT SUPPORTED");
            }
        }

    }

    $Test->nlog("##################################################################################");
    $Test->nlog("##");
    $Test->nlog("## End of POST_TEST logs for Filer : @Filer_array");
    $Test->nlog("##");
    $Test->nlog("##################################################################################");

}

# Deprecated Subroutine, Delete this in 5.2.1
####################################################################################################
# ZNS : Identify commands
#
# Burt :
# Author : arunak
####################################################################################################
sub zns_identify_cmds {
    my (%opts) = validate(
        @_,
        {
            Node_API => { type => OBJECT },
            Drives   => { type => ARRAYREF },
            Handle   => { type => SCALAR, default => undef, optional => 1 },
        }
    );
    my $Node_API = $opts{Node_API};
    my @drives   = @{ $opts{Drives} };
    my $handle   = $opts{Handle};
    logcomment("SDF : Identify commands for drive @drives ");
    my $pegnvme_drv_details = {};

    #hardcode value  or raed from file.
    # New Standard SDF Identify Command
    foreach my $drive_id (@drives) {
        logcomment("Executing identify command for drive $drive_id");
        my $output = $Node_API->execute_raw_command( 'command' => "sudo peg_nvmecontrol identify $drive_id" );
        $pegnvme_drv_details->{$drive_id}->{'pegdrive'} = $drive_id;
        my ($id) = $drive_id =~ /peg_nvme(\d+)/;
        $pegnvme_drv_details->{$drive_id}->{'id'} = $id;
        my $zns_vendor_name;
        foreach my $line ( split( /\n/, $output ) ) {
            if ( $line =~ /Vendor ID:\s+(\S+)/ ) {
                my $id_vendor = $1;
                $pegnvme_drv_details->{$drive_id}->{'id_vendor'} = $id_vendor;
                if ( $id_vendor eq "144d" ) {
                    $zns_vendor_name                                       = "samsung";
                    $pegnvme_drv_details->{$drive_id}->{'zns_vendor_name'} = $zns_vendor_name;
                    $pegnvme_drv_details->{$drive_id}->{'cns_nsp'}         = $drive_id . "ns2";
                    $pegnvme_drv_details->{$drive_id}->{'zone_nsp'}        = $drive_id . "ns1";
                } elsif ( $id_vendor eq "1b96" ) {
                    $zns_vendor_name                                       = "wdc";
                    $pegnvme_drv_details->{$drive_id}->{'zns_vendor_name'} = $zns_vendor_name;
                    $pegnvme_drv_details->{$drive_id}->{'cns_nsp'}         = $drive_id . "ns1";
                    $pegnvme_drv_details->{$drive_id}->{'zone_nsp'}        = $drive_id . "ns2";
                }
            }
            if ( $line =~ /Subsystem Vendor ID:\s+(\S+)/ ) {
                $pegnvme_drv_details->{$drive_id}->{'id_subsystem'} = $1;
            }
            if ( $line =~ /Serial Number:\s+(\S+)/ ) {
                $pegnvme_drv_details->{$drive_id}->{'id_sn'} = $1;
            }
            if ( $line =~ /Model Number:\s+(\S+)/ ) {
                $pegnvme_drv_details->{$drive_id}->{'id_mdl'} = $1;
            }
            if ( $line =~ /Firmware Version:\s+(\S+)/ ) {
                $pegnvme_drv_details->{$drive_id}->{'id_fw'} = $1;
            }
            if ( $line =~ /Total NVM Capacity:\s+(\d+)/ ) {
                $pegnvme_drv_details->{$drive_id}->{'id_nvm_cap'} = $1;
            }
        }

        #NSID_ZONE
        logcomment("Executing Zone namespace identify command");
        my $zonenspdrv = $pegnvme_drv_details->{$drive_id}->{'zone_nsp'};
        my ( $nspid, $nsid_zone_ns ) = $zonenspdrv =~ /peg_nvme(\d+)(ns\d)/;
        $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'pegdrive_nsp'} = $zonenspdrv;
        $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'nspid'}        = $nspid;
        $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'nsid_zone_ns'} = $nsid_zone_ns;
        my $output = $Node_API->execute_raw_command( 'command' => "sudo peg_nvmecontrol identify -s zoned $zonenspdrv" );
        my ( $zone_sz_in_lba, $zone_cap_lba, $zone_nvm_cap, $zone_mar, $zone_mor, $zone_rwa_sz, $zone_mcsz, $zone_mcsz, $zone_sz, $zone_zrwar );

        foreach my $line ( split( /\n/, $output ) ) {
            if ( $line =~ /Size \(in LBAs\):\s+(\d+)/ ) {
                $zone_sz_in_lba = $1;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'zone_sz_in_lba'} = $zone_sz_in_lba;
            }
            if ( $line =~ /Capacity \(in LBAs\):\s+(\d+)/ ) {
                $zone_cap_lba = $1;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'zone_cap_lba'} = $zone_cap_lba;
            }
            if ( $line =~ /NVM Capacity:\s+(\d+)/ ) {
                $zone_nvm_cap = $1;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'zone_nvm_cap'} = $zone_nvm_cap;
            }
            if ( $line =~ /Maximum Active Resources:\s+(\d+)/ ) {
                $zone_mar = $1;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'zone_mar'} = $zone_mar;
            }
            if ( $line =~ /Maximum Open Resources:\s+(\d+)/ ) {
                $zone_mor = $1;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'zone_mor'} = $zone_mor;
            }
            if ( $line =~ /Maximum Zone RWA Resources:\s+(\d+)/ ) {
                $zone_zrwar = $1;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'zone_zrwar'} = $zone_zrwar;
            }
            if ( $line =~ /Zone RWA Size:\s+(\d+)/ ) {
                $zone_rwa_sz = $1;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'zone_rwa_sz'} = $zone_rwa_sz;
            }
            if ( $line =~ /Zone RWA Commit Granularity:\s+(\d+)/ ) {
                $zone_mcsz = $1;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'zone_mcsz'} = $zone_mcsz;

                # temp workaround since Commit size entry is not correct
                if ( $zns_vendor_name eq "wdc" ) {
                    $zone_mcsz = 32;
                    $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'zone_mcsz'} = $zone_mcsz;
                }
            }
            if ( $line =~ /Zone Size:\s+(\d+)/ ) {
                $zone_sz = $1;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'zone_sz'} = $zone_sz;

                #
                # Now calculate needed variables from Zoned data
                #
                my ( $rwa_4k_blks_user, $rwa_4k_blks_total, $rwa_sz_cap, $zone_4k_blks_user, $zone_4k_blks_total );
                if ( $zns_vendor_name eq "samsung" ) {
                    $rwa_4k_blks_user  = $zone_rwa_sz / 4096;
                    $rwa_4k_blks_total = $zone_rwa_sz / 4096;
                    $rwa_sz_cap        = $zone_rwa_sz / 2**10;

                    $zone_4k_blks_user  = $zone_sz / 4096;
                    $zone_4k_blks_total = $zone_sz / 4096;
                } elsif ( $zns_vendor_name eq "wdc" ) {
                    $rwa_4k_blks_user  = $zone_rwa_sz / 4096;
                    $rwa_4k_blks_total = $zone_sz / 4096;
                    $rwa_sz_cap        = $zone_rwa_sz / 2**20;

                    $zone_4k_blks_user  = $zone_rwa_sz / 4096;
                    $zone_4k_blks_total = $zone_sz / 4096;
                }

                my $rwa_mcsz_entries = $rwa_4k_blks_user / $zone_mcsz;

                my $total_zones       = $zone_cap_lba / $zone_4k_blks_user;
                my $zone_mcsz_entries = $zone_4k_blks_user / $zone_mcsz;
                my $total_1mb_zone    = $zone_4k_blks_user / 256;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'total_zones'}        = $total_zones;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'total_1mb_zone'}     = $total_1mb_zone;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'zone_mcsz_entries'}  = $zone_mcsz_entries;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'rwa_mcsz_entries'}   = $rwa_mcsz_entries;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'rwa_4k_blks_user'}   = $rwa_4k_blks_user;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'rwa_4k_blks_total'}  = $rwa_4k_blks_total;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'rwa_sz_cap'}         = $rwa_sz_cap;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'zone_4k_blks_user'}  = $zone_4k_blks_user;
                $pegnvme_drv_details->{$drive_id}->{$zonenspdrv}->{'zone_4k_blks_total'} = $zone_4k_blks_total;
            }
        }

        #NSID_CNS
        my $cnsnspdrv = $pegnvme_drv_details->{$drive_id}->{'cns_nsp'};
        my ( $nspid, $nsid_zone_ns ) = $cnsnspdrv =~ /peg_nvme(\d+)(ns\d)/;
        $pegnvme_drv_details->{$drive_id}->{$cnsnspdrv}->{'pegdrive_nsp_cns'} = $cnsnspdrv;
        $pegnvme_drv_details->{$drive_id}->{$cnsnspdrv}->{'nspid'}            = $nspid;
        $pegnvme_drv_details->{$drive_id}->{$cnsnspdrv}->{'nsid_zone_ns'}     = $nsid_zone_ns;
        my $output = $Node_API->execute_raw_command( 'command' => "sudo peg_nvmecontrol identify $cnsnspdrv" );
        my ( $cns_sz_lba, $cns_cap_lba, $cns_nvm_cap );

        foreach my $line ( split( /\n/, $output ) ) {
            if ( $line =~ /Size \(in LBAs\):\s+(\d+)/ ) {
                $cns_sz_lba = $1;
                $pegnvme_drv_details->{$drive_id}->{$cnsnspdrv}->{'cns_sz_lba'} = $cns_sz_lba;
            }
            if ( $line =~ /Capacity \(in LBAs\):\s+(\d+)/ ) {
                $cns_cap_lba = $1;
                $pegnvme_drv_details->{$drive_id}->{$cnsnspdrv}->{'cns_cap_lba'} = $cns_cap_lba;
            }
            if ( $line =~ /NVM Capacity:\s+(\d+)/ ) {
                $cns_nvm_cap = $1;
                $pegnvme_drv_details->{$drive_id}->{$cnsnspdrv}->{'cns_nvm_cap'} = $cns_nvm_cap;

                #
                # Now calculate needed variables from CNS data
                #
                my $io_4k        = 1;
                my $io_256k      = 64;
                my $io_1m        = 256;
                my $cns_4k_cnt   = $cns_cap_lba / $io_4k;
                my $cns_256k_cnt = $cns_cap_lba / $io_256k;
                my $cns_1m_cnt   = $cns_cap_lba / $io_1m;
                $pegnvme_drv_details->{$drive_id}->{$cnsnspdrv}->{'io_4k'}        = $io_4k;
                $pegnvme_drv_details->{$drive_id}->{$cnsnspdrv}->{'io_256k'}      = $io_256k;
                $pegnvme_drv_details->{$drive_id}->{$cnsnspdrv}->{'io_1m'}        = $io_1m;
                $pegnvme_drv_details->{$drive_id}->{$cnsnspdrv}->{'cns_4k_cnt'}   = $cns_4k_cnt;
                $pegnvme_drv_details->{$drive_id}->{$cnsnspdrv}->{'cns_256k_cnt'} = $cns_256k_cnt;
                $pegnvme_drv_details->{$drive_id}->{$cnsnspdrv}->{'cns_1m_cnt'}   = $cns_1m_cnt;
            }
        }
    }
    logcomment("Completed executing identify commands on all dirves");
    return $pegnvme_drv_details;
}

sub disp_pegnvme_drive_variables {
    my (%opts) = validate(
        @_,
        {
            handle                => { type => SCALAR, default => undef, optional => 1 },
            peg_nvme_hash_details => { type => HASHREF },
            Test                  => { type => OBJECT },
        }
    );
    my $handle               = $opts{handle};
    my $Test                 = $opts{Test};
    my $peg_nvme_all_details = $opts{peg_nvme_hash_details};
    foreach my $peg_drv_name ( keys %$peg_nvme_all_details ) {
        my $zns_vendor_name = $peg_nvme_all_details->{$peg_drv_name}->{'zns_vendor_name'};
        my ( $peg_nvme_ns1, $peg_nvme_ns2 );
        if ( $zns_vendor_name eq 'samsung' ) {
            $peg_nvme_ns1 = $peg_nvme_all_details->{$peg_drv_name}->{'zone_nsp'};
            $peg_nvme_ns2 = $peg_nvme_all_details->{$peg_drv_name}->{'cns_nsp'};
        } else {
            $peg_nvme_ns1 = $peg_nvme_all_details->{$peg_drv_name}->{'cns_nsp'};
            $peg_nvme_ns2 = $peg_nvme_all_details->{$peg_drv_name}->{'zone_nsp'};
        }
        logcomment("Displaying Identify commands for drive : $peg_drv_name");
        logcomment("Zone : $peg_nvme_ns1 | CNS : $peg_nvme_ns2");
        $Test->nlog("\n");
        $Test->nlog("=========================================================================================================");
        $Test->nlog("Selected drive = $peg_drv_name -> $zns_vendor_name");
        $Test->nlog("=========================================================================================================");
        $Test->nlog("-----------------------------------------------------------------------------------------");
        $Test->nlog(" IDENTIFY DATA");
        $Test->nlog("-----------------------------------------------------------------------------------------");
        $Test->nlog(" - ID Vendor ID              : $peg_nvme_all_details->{$peg_drv_name}->{'Vendor_ID'}");
        $Test->nlog(" - ID Subsystem ID           : $peg_nvme_all_details->{$peg_drv_name}->{'Subsystem_Vendor_ID'}");
        $Test->nlog(" - ID Serno                  : $peg_nvme_all_details->{$peg_drv_name}->{'Serial_Number'}");
        $Test->nlog(" - ID Model                  : $peg_nvme_all_details->{$peg_drv_name}->{'Model_Number'}");
        $Test->nlog(" - ID Firmware               : $peg_nvme_all_details->{$peg_drv_name}->{'Firmware_Version'}");
        $Test->nlog(" - ID NVM cap                : $peg_nvme_all_details->{$peg_drv_name}->{'Total_NVM_Capacity'}");
        $Test->nlog(" - ID Zone namespace         : $peg_nvme_all_details->{$peg_drv_name}->{'zone_nsp'}");
        $Test->nlog(" - ID CNS namespace          : $peg_nvme_all_details->{$peg_drv_name}->{'cns_nsp'}");
        $Test->nlog("-----------------------------------------------------------------------------------------");
        $Test->nlog(" IDENTIFY ZONED DATA : $zns_vendor_name");
        $Test->nlog("-----------------------------------------------------------------------------------------");
        $Test->nlog(" - Zone sz in LBA            : $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns1}->{'Size_(in_LBAs)'}");
        $Test->nlog(" - Zone cap in LBA           : $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns1}->{'Capacity_(in_LBAs)'}");
        $Test->nlog(" - Zone nvm CAP              : $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns1}->{'NVM_Capacity'}");
        $Test->nlog(" - Zone Max Active resources : $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns1}->{'Maximum_Active_Resources'}");
        $Test->nlog(" - Zone Max Open resources   : $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns1}->{'Maximum_Open_Resources'}");
        $Test->nlog(" - Zone RWA resources        : $peg_nvme_all_details->{$peg_drv_name}->{'Maximum_Random_Write_Zones'}");
        $Test->nlog(" - Zone Commit sz            : $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns1}->{'Random_Write_Area_Commit_Granularity'}");
        $Test->nlog(" - Zone Size (cap)           : $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns1}->{'LBA_Format_0_Extension_Zone_Size'}");
        $Test->nlog("\n");
        my $val = $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns1}->{'Random_Write_Area_Size'} / 2**10 / 4;
        $Test->nlog(" * ZRWA 4k blks (total)      : $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns1}->{'Random_Write_Area_Size'} = $val");
        $Test->nlog(" * ZONE 4k blks (total)      : $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns1}->{'LBA_Format_1_Extension_Zone_Size'}");
        $Test->nlog(" * ZONE Total Zones          : $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns1}->{'Capacity_(in_LBAs)'}");
        $Test->nlog("-----------------------------------------------------------------------------------------");
        $Test->nlog(" IDENTIFY CNS DATA : $zns_vendor_name");
        $Test->nlog("-----------------------------------------------------------------------------------------");
        my ($siz) = $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns2}->{'Size_(in_LBAs)'} =~ /(\S+)\_(.*)/;
        my $cns_nvm_cap = ( $siz * 4096 / 2**10 );
        my $cns_32kblks = $siz / 32;
        my $cns_64kblks = $siz / 64;
        $Test->nlog(" - CNS sz in LBA             : $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns2}->{'Size_(in_LBAs)'}");
        $Test->nlog(" - CNS nvm cap               : $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns2}->{'Capacity_(in_LBAs)'}");
        $Test->nlog(" - CNS nvm cap               : $cns_nvm_cap");
        $Test->nlog(" * CNS 4k blks               : $peg_nvme_all_details->{$peg_drv_name}->{$peg_nvme_ns2}->{'Size_(in_LBAs)'}");
        $Test->nlog(" * CNS 32k blks		: $cns_32kblks");
        $Test->nlog(" * CNS 64k blks              : $cns_64kblks");
        $Test->nlog("\n");
        $Test->nlog("=========================================================================================================");

    }

    #logcomment( "Dumper " . Dumper($peg_nvme_all_details) );
}
####################################################################################################
# NVME - Eye Catcher to be called in all NVMe-Scripts
# Return : N/A
# Burt :
# Author : arunak
####################################################################################################
sub zns_eye_catcher {
    my (%opts) = validate(
        @_,
        {
            Test   => { type => OBJECT },
            string => { type => SCALAR, default => undef, optional => 1 },
        }
    );
    my $Test   = $opts{Test};
    my $string = $opts{string};
    $Test->nlog("###################################################################################");
    $Test->nlog("### SDF : $string");
    $Test->nlog("###################################################################################");
}

sub random_num_gen_min2max {
    my (%opts) = validate(
        @_,
        {
            min => { type => SCALAR, default => 0,   optional => 1 },
            max => { type => SCALAR, default => 100, optional => 1 },
        }
    );
    my $min = $opts{min};
    my $max = $opts{max};
    $max = $max + 1;
    return $min + int( rand( $max - $min ) );

}

sub get_pegnvme_from_file {
    my (%opts) = validate( @_, { handle => { type => SCALAR, default => undef, optional => 1 }, } );
    my $handle = $opts{handle};
    my $Home   = $ENV{HOME};
    my $file   = $Home . "/zns_drive_list";
    logcomment("Get drive from : $file");
    open( FH, $file );
    chomp( my @drives = <FH> );
    if (@drives) {
        logcomment("Drives captured from file : @drives\n");
    } else {
        logcomment("Drives details in $file appears to be in wrong format\n");
        return 1;
    }
    return \@drives;
}

#ZNS SETUP_CMODE SUBROUTINE
sub setup_cmode_zns {
    my (%opts) = validate(
        @_,
        {
            node_present => { type => ARRAYREF },
            Test         => { type => OBJECT },
            event_time   => { type => SCALAR, optional => 1, default => 'NO' },
        }
    );
    my @Nodes      = @{ $opts{node_present} };
    my $Test       = $opts{Test};
    my $event_time = $opts{event_time};
    logcomment("Node Present :");
    foreach my $Node (@Nodes) {
        my $FILER_C = $Node->name();
        logcomment(" $FILER_C");
    }
    logcomment("DEBUG:Setup Cmode filer");
    my $Home = $ENV{HOME};
    my @subtests;
    my $prompt_fw;
    my $Transit_obj;
    my $Server = new Server( no_cli => 1 );

    # Check state and boot in maintenancae
    my @change_2_maint = ();

    foreach my $Node (@Nodes) {
        my $FILER_C    = $Node->name();
        my $filer_info = `cat $Home/NDATE/FILER_INFO/$FILER_C`;
        my $passwd;
        ($passwd) = $filer_info =~ /default_password\s*=\s*(\S+)\s*/ig;
        $Server->set_execution_server( -Server => $FILER_C );
        $Transit_obj = NACL::Transit->new( name => $FILER_C );
        my $filer_state    = $Transit_obj->get_state( 'timeout' => 3600, 'get_state_timeout' => 7200 );
        my $timeout        = "1900";
        my $ignore_strings = [ "MADIMM\\d+->", "DIMM\\d+->", "^T>" ];
        logcomment("Filer $FILER_C is in $filer_state state");

        if ( $filer_state =~ /CLI|UP/i ) {
            try {
                $Server->console(
                    cmd                        => "node rename -newname $FILER_C",
                    additional_prompts_timeout => $timeout,
                    ignore                     => $ignore_strings,
                    additional_prompts         => {
                        "$prompt_fw"   => "FINAL_PROMPT",
                        "::>"          => "FINAL_PROMPT",
                        "login"        => "admin",
                        ".*Password.*" => "$passwd",
                    }
                );
                logcomment("Changed node name");
            }
            catch NACL::Transit::Exceptions::TransitException with {
                my $exception_object = shift;
                logcomment( "Exception is of type: " . ref($exception_object) );
                logcomment( "Error text is: " . $exception_object->text() );
            };
        }
        if ( $filer_state !~ /CLI|UP|MAINT/i ) {
            $Transit_obj->set( 'change_state_timeout' => 7200 );
            $Transit_obj->change_state( to => "MAINT" );
        }
    }
    logcomment("Set boot args for ZNS drive visibility");
    set_zns_boot_args( node_present => \@Nodes );
    logcomment("Boot arg set on both nodes, Proceed with test");
    logcomment( "Wipe config for the filers will be initiated" . scalar( localtime() ) );
    if ( $event_time =~ /NO/ ) {
        foreach my $Node (@Nodes) {
            my $filer_name = $Node->name();
            push( @subtests, subtest( \&wipe_config_zns, -runid, "wipe_config_filers_" . "$filer_name", -bg, "--", node => $Node, ) );
        }
    } else {
        foreach my $Node (@Nodes) {
            my $filer_name = $Node->name();
            push(
                @subtests,
                subtest(
                    \&wipe_config_zns,                     -runid,
                    "wipe_config_filers_" . "$filer_name", -bg,
                    "--",
                    node       => $Node,
                    event_time => $event_time,
                )
            );
        }
    }
    Subtest::wait_finish( subtest => [@subtests] );
    logcomment( "Wipe config completed" . scalar( localtime() ) );

    my $filer_state;
    foreach my $Node (@Nodes) {
        my $FILER_C;
        try {
            $FILER_C = $Node->name();
            logcomment("DEBUG:Cheking the state of filer $FILER_C");
            my $Transit_obj = NACL::Transit->new( name => $FILER_C );
            $filer_state = $Transit_obj->get_state( 'timeout' => 3600, 'get_state_timeout' => 7200 );
            logcomment("Filer : $FILER_C is in $filer_state state");
            if ( $filer_state !~ /CLI/i ) {
                $Transit_obj->set( 'change_state_timeout' => 7200 );
                $Transit_obj->change_state( to => "CLI" );
                $filer_state = $Transit_obj->get_state( 'timeout' => 3600, 'get_state_timeout' => 7200 );
                logcomment("DEBUG:Filer : $FILER_C has changed state to $filer_state");
            }

        }
        catch NACL::Transit::Exceptions::TransitException with {
            logcomment("Caught TransitException exception");
            my $exception_object = shift;
            logcomment("**WARNING** : Filer - $FILER_C : is in $filer_state state");
        };
    }

    logcomment("Node Present :");
    foreach my $Node (@Nodes) {
        my $FILER_C = $Node->name();
        logcomment(" $FILER_C");
    }
    @Nodes = sort(@Nodes);
    logcomment("Node sorted :");
    foreach my $Node (@Nodes) {
        my $FILER_C = $Node->name();
        logcomment(" $FILER_C");
    }
    foreach my $a (@Nodes) {
        my $fil_name = $a->name();
        logcomment("DEBUG:Node name is - $fil_name");
    }
    logcomment("Creating cluster using nodes available");
    create_join_cluster( node_present => [@Nodes] );
    logcomment("Completed creating and joining cluster ");
    my @temp_nodes = NACL::C::Node->find();    # Find Nodes/Filers used in the test, Based on FILER param.
    my %nodes_filer;
    foreach my $Node (@temp_nodes) {
        my $FILER_C = $Node->name();
        $nodes_filer{$FILER_C} = [$Node];
    }
    my @Nodes_all = map { @{$_} } grep { (/\S/) } values(%nodes_filer);    # Contains Node object used for test execution.
    foreach my $Node (@Nodes_all) {
        my $FILER_C = $Node->name();
        my $Home    = $ENV{HOME};
        system("rm -rf $Home/.ssh/known_hosts");
        $Home = trim($Home);
        my $host_file    = "$Home/NDATE/FILER_INFO/$FILER_C";
        my $filer_config = `cat $host_file`;
        my $def_pass;
        foreach my $line ( split /\n/, $filer_config ) {

            if ( $line =~ /^default_password=(.*)/ ) {
                $def_pass = $1;
                logcomment("$FILER_C : Changing the diag mode password to $def_pass");
                last;
            }
        }
        try {
            my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset();
            $Api_Set_Obj->set_timeout( "connectrec-timeout" => 7200 );
            $Api_Set_Obj->execute_raw_command( 'command' => "security login unlock -username diag" );
            my $prompts_answers = [ 'Enter a new password:' => "$def_pass", 'Enter it again:' => "$def_pass" ];
            $Api_Set_Obj->execute_raw_command( 'command' => "security login password -username diag", 'connectrec-match_table' => $prompts_answers );
            $Api_Set_Obj->execute_raw_command( 'command' => "version" );
            my $prompts_answers = [ ".*Do you want to continue.*" => 'y' ];
            $Api_Set_Obj->execute_raw_command( 'command' => "set test", 'connectrec-match_table' => $prompts_answers );
        }
        catch NATE::BaseException with {
            logcomment("$FILER_C : Caught BASE EXCEPTION, connection problem!!!");
        };
    }
}

sub READ_SMART {
    my (%opts) = validate(
        @_,
        {
            API_Obj => { type => OBJECT },
            peg_id  => { type => SCALAR },
            filer   => { type => SCALAR },
        }
    );
    my $Api_Set_Obj_SS  = $opts{API_Obj};
    my $peg_id          = $opts{peg_id};
    my $read_smart_hash = {};
    my $filer           = $opts{filer};
    my $out             = $Api_Set_Obj_SS->execute_raw_command( 'command' => " sudo peg_nvmecontrol logpage -p 2 $peg_id" );
    foreach my $line ( split( /\n/, $out ) ) {
        if ( $line =~ /Data\s+units\s+\(\d+\s+\S+\)\s+read\:\s+(\S+)/ ) {
            $read_smart_hash->{$peg_id}->{$filer}->{"SMART_DATA_UNITS_READ"} = $1;
        } elsif ( $line =~ /Data\s+units\s+\(\d+\s+\S+\)\s+written\:\s+(\S+)/ ) {
            $read_smart_hash->{$peg_id}->{$filer}->{"SMART_DATA_UNITS_WRITE"} = $1;
        } elsif ( $line =~ /Host\s+read\s+commands\:\s+(\S+)/ ) {
            $read_smart_hash->{$peg_id}->{$filer}->{"SMART_HOST_CMD_READ"} = $1;
        } elsif ( $line =~ /Host\s+write\s+commands\:\s+(\S+)/ ) {
            $read_smart_hash->{$peg_id}->{$filer}->{"SMART_HOST_CMD_WRITE"} = $1;
        } elsif ( $line =~ /Power\s+cycles\:\s+(\S+)/ ) {
            $read_smart_hash->{$peg_id}->{$filer}->{"SMART_POWER_CYCLES"} = $1;
        } elsif ( $line =~ /Media\s+errors\:\s+(\S+)/ ) {
            $read_smart_hash->{$peg_id}->{$filer}->{"SMART_MEDIA_ERRORS"} = $1;
        } elsif ( $line =~ /No.\s+error\s+info\s+log\s+entries\:\s+(\S+)/ ) {
            $read_smart_hash->{$peg_id}->{$filer}->{"SMART_ERR_LOG_ENTRIES"} = $1;
        }
    }

=head
	logcomment("SMART_DATA_UNITS_READ: ",$read_smart_hash->{"SMART_DATA_UNITS_READ"});
	logcomment("SMART_DATA_UNITS_WRITE: ",$read_smart_hash->{"SMART_DATA_UNITS_WRITE"});
	logcomment("SMART_HOST_CMD_READ: ",$read_smart_hash->{"SMART_HOST_CMD_READ"});
	logcomment("SMART_HOST_CMD_WRITE: ",$read_smart_hash->{"SMART_HOST_CMD_WRITE"});
	logcomment("SMART_POWER_CYCLES: ",$read_smart_hash->{"SMART_POWER_CYCLES"});
	logcomment("SMART_MEDIA_ERRORS: ",$read_smart_hash->{"SMART_MEDIA_ERRORS"});
	logcomment("SMART_ERR_LOG_ENTRIES: ",$read_smart_hash->{"SMART_ERR_LOG_ENTRIES"});
=cut	

    return $read_smart_hash;
}

sub READ_LINK_STATUS {
    my (%opts) = validate(
        @_,
        {
            API_Obj => { type => OBJECT },
            peg_id  => { type => SCALAR },
        }
    );
    my $Api_Set_Obj      = $opts{API_Obj};
    my $read_link_status = {};
    my $peg_id           = $opts{peg_id};
    my ($drv_id) = ( $peg_id =~ /peg_nvme(\d+)/ );
    my $out = $Api_Set_Obj->execute_command( 'command' => "storage show" );
    foreach my $line ( split /\n/, $out ) {
        next if ( $line =~ /disk|^-|displayed|debug|entries|^$|session\.change/ );
        if ( $line =~ /\[.*]/ ) {
            if ( $line =~ /\[\s*($drv_id)\s*\]\s+(DIS\/LNK|LNK|DIS)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/i ) {
                my ( $drv, $state, $lnk_lan, $max_lnk, $neg_lnk, $med_mx_lnk, $dev_mx_la, $Neg_max, $lan_wd ) = ( $line =~ /\[\s*($drv_id)\s*\]\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/i );
                logcomment("**WARNING**:Disk state is -$state for drive-$drv");
                $read_link_status->{"$peg_id"}->{'state'}          = $state;
                $read_link_status->{"$peg_id"}->{'neg_link_speed'} = $neg_lnk;
                $read_link_status->{"$peg_id"}->{'neg_lane_width'} = $lan_wd;

            } elsif ( $line =~ /\[\s*($drv_id)\s*\]\s+(OK)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/i ) {
                my ( $drv, $state, $lnk_lan, $max_lnk, $neg_lnk, $med_mx_lnk, $dev_mx_la, $Neg_max, $lan_wd ) = ( $line =~ /\[\s*($drv_id)\s*\]\s+(OK)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/i );
                unless ( $neg_lnk == 8 ) {
                    logcomment("**WARNING**: Negotiated link speed is -$neg_lnk, it Must be '8.0' for drive -$drv");
                    $read_link_status->{"$peg_id"}->{'state'}          = $state;
                    $read_link_status->{"$peg_id"}->{'neg_link_speed'} = $neg_lnk;
                    $read_link_status->{"$peg_id"}->{'neg_lane_width'} = $lan_wd;
                } else {
                    $read_link_status->{"$peg_id"}->{'state'}          = $state;
                    $read_link_status->{"$peg_id"}->{'neg_link_speed'} = $neg_lnk;
                    $read_link_status->{"$peg_id"}->{'neg_lane_width'} = $lan_wd;
                }
                unless ( $lan_wd == 2 ) {
                    logcomment("**WARNING**: Negotiated lane width is -$lan_wd, it Must be '2' for drive-$drv");
                    $read_link_status->{"$peg_id"}->{'state'}          = $state;
                    $read_link_status->{"$peg_id"}->{'neg_link_speed'} = $neg_lnk;
                    $read_link_status->{"$peg_id"}->{'neg_lane_width'} = $lan_wd;
                } else {
                    $read_link_status->{"$peg_id"}->{'state'}          = $state;
                    $read_link_status->{"$peg_id"}->{'neg_link_speed'} = $neg_lnk;
                    $read_link_status->{"$peg_id"}->{'neg_lane_width'} = $lan_wd;
                }
            } elsif ( $line =~ /\[\s*($drv_id)\s*\]\s+(EMPTY)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/i ) {
                my ( $drv, $state, $lnk_lan, $max_lnk, $neg_lnk, $med_mx_lnk, $dev_mx_la, $Neg_max, $lan_wd ) = ( $line =~ /\[\s*($drv_id)\s*\]\s+(EMPTY)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/i );

                #=head
                $read_link_status->{"$peg_id"}->{'state'}          = $state   unless ( defined $read_link_status->{"$peg_id"}->{'state'} );
                $read_link_status->{"$peg_id"}->{'neg_link_speed'} = $neg_lnk unless ( defined $read_link_status->{"$peg_id"}->{'neg_link_speed'} );
                $read_link_status->{"$peg_id"}->{'neg_lane_width'} = $lan_wd  unless ( defined $read_link_status->{"$peg_id"}->{'neg_lane_width'} );

                #=cut
                next;
            }
        }

    }
    return $read_link_status;
}

sub READ_SKTRACE {
    my (%opts) = validate(
        @_,
        {
            API_Obj => { type => OBJECT },
            peg_id  => { type => SCALAR },
            filer   => { type => SCALAR },
            cmd     => { type => SCALAR },
        }
    );
    my $Api_Set_Obj_SS  = $opts{API_Obj};
    my $peg_id          = $opts{peg_id};
    my $sktrace_err_cnt = {};
    my $filer           = $opts{filer};
    my $cmd             = $opts{cmd};
    my @arr             = ();
    my $out             = $Api_Set_Obj_SS->execute_raw_command( 'command' => "$cmd" );

    if ( $out =~ /Subsystem\s+reset\s+initiated/i ) {
        foreach my $line ( split( /\n/, $out ) ) {
            if ( $line =~ /((\S+)\s+(\[\d+:\d+\])?\s+\S+\:\s+\S+\s+$peg_id\:\s+Subsystem\s+reset\s+initiated)/i ) {
                push( @arr, $2 );
            }
        }
        $sktrace_err_cnt->{$peg_id}->{$filer} = @arr[-1];
    } elsif ( $out =~ /Fundamental\s+reset\s+initiated/i ) {
        foreach my $line ( split( /\n/, $out ) ) {
            if ( $line =~ /((\S+)\s+(\[\d+:\d+\])?\s+\S+\:\s+\S+\s+$peg_id\:\s+Fundamental\s+reset\s+initiated)/i ) {
                push( @arr, $2 );
            }
        }
        $sktrace_err_cnt->{$peg_id}->{$filer} = @arr[-1];
    } elsif ( $out =~ /resetting\s+controller/i ) {
        foreach my $line ( split( /\n/, $out ) ) {
            if ( $line =~ /((\S+)\s+(\[\d+:\d+\])?\s+\S+\:\s+\S+\s+$peg_id\:\s+resetting\s+controller)/i ) {
                push( @arr, $2 );
            }
        }
        $sktrace_err_cnt->{$peg_id}->{$filer} = @arr[-1];
    } else {
        logcomment("**WARNING**: Error  value not appeared");
        return;
    }
    return $sktrace_err_cnt;
}

##Wipe config ZNS filers
sub wipe_config_zns {
    my (%opts) = validate(
        @_,
        {
            node         => { type => OBJECT },
            event_time   => { type => SCALAR, optional => 1, default => 'NO' },
            zeroing_type => { type => SCALAR, optional => 1, default => 'LOGICAL' },
        }
    );
    my $Node         = $opts{node};
    my $event_time   = $opts{event_time};
    my $Image        = $opts{image};
    my $zeroing_type = $opts{zeroing_type};
    my $Home         = $ENV{HOME};
    my $Node_name    = $Node->name();
    my $filer_info   = `cat $Home/NDATE/FILER_INFO/$Node_name`;
    my $passwd;
    ($passwd) = $filer_info =~ /default_password\s*=\s*(\S+)\s*/ig;
    logcomment("DEBUG:password is $passwd");
    #############################################################################

    if ( $event_time !~ /NO/ ) {
        logcomment( scalar( localtime() ) . " Starting for $event_time time" );
    }
    my $time    = get_max_timeout( node_present => [$Node] );
    my $Host    = host($Node_name);
    my $API_OBJ = NACL::APISet->new(
        hostobj   => $Host,
        category  => "Node",
        interface => "CLI",
        set       => "Maintenance"
    );
    $API_OBJ->set_timeout( "connectrec-timeout" => 7200 );
    my $disk_tot      = disk_show_drive_maint( API_Obj => $API_OBJ, Filer => $Node_name );
    my @drives        = @{ $disk_tot->{'driveA'} };
    my $total_tomeout = ( $time * ( scalar @drives ) );

    #############################################################################
    my $prompt_fw;
    my $wipeclean_timeout = 660;
    my $ignore_strings = [ "MADIMM\\d+->", "DIMM\\d+->", "^T>" ];

    # Retreive valid prompts from NATE
    my $host         = Tharn::host($Node_name);
    my $prompt_array = $host->prompt_fw();
    my $prompts      = join( "|", @$prompt_array );
    $prompt_fw = "($prompts)";
    logcomment("DEBUG:This is the prompt regex: $prompt_fw");
    logcomment("Disable cluster/HA pair");
    my $Server = new Server( no_cli => 1 );
    $Server->set_execution_server( -Server => $Node_name );

    #    $Server->console(cmd => "storage failover modify -node * -enabled false -auto-giveback false" );
    #    logcomment("Enabled  storage failover");

    my $Transit_obj = NACL::Transit->new( name => $Node_name );
    my $conn = $Transit_obj->{'conn'};
    $Transit_obj->set( 'timeout' => 600, maxcycle => 20 );
    $Transit_obj->change_state( to => "FIRMWARE" );
    my $Server = new Server( no_cli => 1 );
    logcomment("Set AUTO BOOT to false ");
    my $timeout = "9000";
    $Server->set_execution_server( -Server => $Node_name );
    $Server->console(
        cmd                        => "setenv AUTOBOOT false",
        additional_prompts_timeout => $timeout,
        ignore                     => $ignore_strings,
        additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT" }
    );
    logcomment("AUTOBOOT set to false");
    $Server->console(
        cmd                        => "printenv",
        additional_prompts_timeout => $timeout,
        ignore                     => $ignore_strings,
        additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT" }
    );

    #Below change is made according to Burt No: 1312736
    if ( $zeroing_type =~ /PHYSICAL/ig ) {
        logcomment("Set fast-zeroing to false");
        $Server->console(
            cmd                        => "setenv raid.enable-fast-zeroing? false",
            additional_prompts_timeout => $timeout,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT" }
        );
    } else {
        logcomment("Set fast-zeroing to true");
        $Server->console(
            cmd                        => "setenv raid.enable-fast-zeroing? true",
            additional_prompts_timeout => $timeout,
            ignore                     => $ignore_strings,
            additional_prompts         => { "$prompt_fw" => "FINAL_PROMPT" }
        );
    }
    sleep(30);
    my $timeout = "3600";    # Change timeout in final code.
    $Server->console(
        cmd                        => "boot_ontap",
        additional_prompts_timeout => $timeout,
        ignore                     => $ignore_strings,
        additional_prompts         => {
            ".* Press Ctrl-C for Boot Menu.*"                => "\003",
            ".*Clean configuration and initialize all disks" => "FINAL_PROMPT",
            ".*LOADER-.*"                                    => "FATAL_PROMPT",
        }
    );

    #$Transit_obj->set( 'change_state_timeout' => 7200 );
    #    $Transit_obj->change_state( to => "FIRMWARE" );
    logcomment("DEBUG:$Node_name : Filer is in boot_state");
    logcomment("DEBUG:$Node_name : Creating Server object");
    my $Server = new Server( no_cli => 1 );
    my @filer_working = $Server->servers();
    logcomment("DEBUG:Filers we are working on : @filer_working");

    my $timeout = "90000";    # Change timeout in final code.
    $timeout = $total_tomeout if ( $total_tomeout > $timeout );
    my ( $s, $m, $h ) = gmtime($timeout);
    logcomment("DEBUG:Setting timeout : $timeout seconds (i.e) $h Hours $m Minutes $s Seconds");
    logcomment("DEBUG:Set execution server");
    $Server->set_execution_server( -Server => $Node_name );    #set the execution server to the current node
    logcomment( "Boot_menu wipe clean util called " . scalar( localtime() ) );
    logcomment("Execution server set to $Node_name");
    logcomment("DEBUG:Disable password ");
    $Server->console( cmd => "options security.passwd.rules.everyone off", noprompt => 1 );
    eval {
        $Server->console(
            cmd                        => "4",
            additional_prompts_timeout => $timeout,
            ignore                     => $ignore_strings,
            additional_prompts         => {
                ".*Zero disks, reset config and install a new file system.*" => "yes",
                ".*reset config and install a new file.*"                    => "yes",
                ".*erase all the data on the disks.*"                        => "yes",
                ".*want to continue.*"                                       => "y",
                "Rebooting to finish"                                        => "FATAL_PROMPT",
                ".*Rebooting to finish.*"                                    => "FATAL_PROMPT",
                ".*Rebooting to finish wipeconfig.*"                         => "FATAL_PROMPT",
                "$prompt_fw"                                                 => "FATAL_PROMPT",
                "Selection "                                                 => "FATAL_PROMPT",
                "login"                                                      => "admin",
                ".*Type yes to confirm and continue.*"                       => 'exit',
                "::>"                                                        => "FATAL_PROMPT",
                ".*Password.*"                                               => "$passwd",

            }
        );
    };

    if ($@) {
        if ( $@ !~ /Err=TIMEOUT/ ) {
            if ( $@ =~ /Rebooting to finish/ ) {
                my $Transit_obj = NACL::Transit->new( name => $Node_name );
                logcomment("DEBUG:Changing state to firmware");
                $Server->set_execution_server( -Server => $Node_name );
                $Transit_obj->set( 'change_state_timeout' => 7200 );
                $Transit_obj->change_state( to => "FIRMWARE" );
                my $conn = $Server->get_connection( type => "console_connection" );
                $Server->console(
                    cmd                => "setenv AUTOBOOT false",
                    ignore             => $ignore_strings,
                    additional_prompts => {
                        "$prompt_fw"                           => "FINAL_PROMPT",
                        "::>"                                  => "FINAL_PROMPT",
                        ".*Type yes to confirm and continue.*" => 'exit',
                    }
                );
                sleep 10;
                $Server->console(
                    cmd                => "setenv bootarg.init.boot_clustered true",
                    ignore             => $ignore_strings,
                    additional_prompts => {
                        "$prompt_fw"                           => "FINAL_PROMPT",
                        "::>"                                  => "FINAL_PROMPT",
                        ".*Type yes to confirm and continue.*" => 'exit',
                    }
                );
                $Server->console(
                    cmd                        => "boot_ontap",
                    additional_prompts_timeout => $timeout,
                    ignore                     => $ignore_strings,
                    additional_prompts         => {
                        "$prompt_fw"                                                                       => "FINAL_PROMPT",
                        "::>"                                                                              => "FINAL_PROMPT",
                        "Do you want to create a new cluster or join an existing cluster? {create, join}:" => "exit",
                        "{create, join}:"                                                                  => "exit",
                        ".*create a new cluster or join an existing.*"                                     => "exit",
                        ".*Enter the node management interface IP address.*"                               => "exit",
                        "Enter the node management interface port .*"                                      => "exit",
                        ".*Type yes to confirm and continue.*"                                             => 'exit',
                        ".* Enter the .*"                                                                  => "exit",
                        "Password:"                                                                        => "$passwd",
                        ".*management interface port.*"                                                    => "exit",
                        ".*Welcome to node setup.*"                                                        => "exit",
                        ".*enter the following commands at any time.*"                                     => "exit",
                        ".*To accept a default or omit a question.*"                                       => "exit",
                        ".*interface IP address.*"                                                         => "exit",
                        ".*the node management.*"                                                          => "exit",
                        "login"                                                                            => "admin",
                        ".*Password.*"                                                                     => "$passwd",
                    }
                );
            } else {
                logcomment("Unknown error during wipeconfig :$@");
            }
        } else {
            logcomment("Timeout during wipeconfig:$@");
        }
    }
    logcomment("Renaming node ");
    try {
        $Server->console(
            cmd                        => "node rename -newname $Node_name",
            additional_prompts_timeout => $timeout,
            ignore                     => $ignore_strings,
            additional_prompts         => {
                "$prompt_fw"   => "FINAL_PROMPT",
                "::>"          => "FINAL_PROMPT",
                "login"        => "admin",
                ".*Password.*" => "$passwd",
            }
        );
        logcomment("Changing session timeout Value");
        $Server->console(
            cmd                        => "system timeout modify -timeout 3000",
            additional_prompts_timeout => $timeout,
            ignore                     => $ignore_strings,
            additional_prompts         => {
                "$prompt_fw"   => "FINAL_PROMPT",
                "::>"          => "FINAL_PROMPT",
                "login"        => "admin",
                ".*Password.*" => "$passwd",
            }
        );
    }
    catch NACL::Transit::Exceptions::TransitException with {
        my $exception_object = shift;
        logcomment( "Exception is of type: " . ref($exception_object) );
        logcomment( "Error text is: " . $exception_object->text() );
    };
    logcomment("Checking State of Filer");
    my $Transit_obj    = NACL::Transit->new( name => $Node_name );
    my $conn           = $Transit_obj->{'conn'};
    my $timeout        = "1900";
    my $ignore_strings = [ "MADIMM\\d+->", "DIMM\\d+->", "^T>" ];
    logcomment( "DEBUG:Checking state " . scalar( localtime() ) );
    my $current_state = $Transit_obj->get_state( 'timeout' => 3600, 'get_state_timeout' => 7200 );

    if ( $current_state =~ /USERNAME|PASSWORD/ ) {
        $Transit_obj->set( 'change_state_timeout' => 7200 );
        $Transit_obj->change_state( to => "CLI" );
    }
    logcomment("DEBUG:Verifying Node name ");
    try {
        $Server->console(
            cmd                        => "node rename -newname $Node_name",
            additional_prompts_timeout => $timeout,
            ignore                     => $ignore_strings,
            additional_prompts         => {
                "$prompt_fw"   => "FINAL_PROMPT",
                "::>"          => "FINAL_PROMPT",
                "login"        => "admin",
                ".*Password.*" => "$passwd",
            }
        );
        logcomment("Changed node name");
    }
    catch NACL::Transit::Exceptions::TransitException with {
        my $exception_object = shift;
        logcomment( "Exception is of type: " . ref($exception_object) );
        logcomment( "Error text is: " . $exception_object->text() );
    };
    logcomment("Completed 4d Booting , Proceeding with cluster create");
    if ( $event_time !~ /NO/ ) {
        logcomment("Completed booting for $event_time time");
    }
}

sub zns_copy_FW_files_to_filer_directory {
    my (%opts) = validate(
        @_,
        {
            Node            => { type => OBJECT },
            Files_path      => { type => SCALAR, default => undef, optional => 1 },
            Files           => { type => ARRAYREF, default => undef, optional => 1 },
            DL_PRV_FW_Files => { type => SCALAR, default => undef, optional => 1 },
        }
    );

    logcomment("Inside zns_copy_FW_files_to_filer_directory");
    my $Node                 = $opts{Node};
    my $files                = $opts{Files};
    my $files_path           = $opts{Files_path};
    my $delete_priv_fw_files = $opts{DL_PRV_FW_Files};
    my $FILER_C              = $Node->name();
    my $Api_Set_Obj          = $Node->get_7m_or_nodescope_apiset();
    $Api_Set_Obj->set_timeout( "connectrec-timeout" => 7200 );
    my $Mode = $Node->mode();
    my $Host = host($FILER_C);
    my ( @files_to_remove_dup, @files_to_remove, @files_to_transfer, @fw_files );
    logcomment("Deleting all the previous files present in /mroot/etc/disk_fw directory.") if ( $delete_priv_fw_files =~ /Yes/i );
    logcomment( "Files : ", @{$files} );
    my $Api_Set_Obj_SS = NACL::APISet->new(
        hostobj   => $Host,
        category  => "Node",
        interface => "CLI",
        set       => "Systemshell",
    );
    $Api_Set_Obj_SS->set_timeout( "connectrec-timeout" => 7200 );
    my $fw_path      = "/mroot/etc/disk_fw";
    my $Filer_apiset = $Node->apiset(
        category  => 'Node',
        interface => 'CLI',
        set       => "Systemshell",
    );

    if ( $delete_priv_fw_files =~ /Yes/i ) {
        $Api_Set_Obj_SS->execute_raw_command( command => "sudo chmod 777 /mroot/etc/disk_fw" );
        my $File_on_filer = $Filer_apiset->execute_raw_command( command => "ls /mroot/etc/disk_fw" );
        chomp($File_on_filer);
        my @files_to_remove = grep { /LOD$/ } sort split( /\s+/, $File_on_filer );

        my @actual_disks_t_dup = @files_to_remove;
        my $disk_max_sup       = 3;
        my $str;
        while (@actual_disks_t_dup) {
            my $loop_no = 0;
            my @disk_list_q;
            while ( $loop_no < $disk_max_sup ) {
                push @disk_list_q, pop @actual_disks_t_dup;
                $loop_no++;
                last if ( !(@actual_disks_t_dup) );
            }
            my $local_str = join( " ", @disk_list_q );
            $str = "$str" . "$local_str" . "\n";
        }
        logcomment("Firmware files present under $fw_path  are :\n $str");

        $Api_Set_Obj_SS->execute_raw_command( command => "rm -f /mroot/etc/disk_fw/*" );
    }
    if ( ( defined $files ) && ( !defined $files_path ) ) {
        foreach my $file ( @{$files} ) {
            chomp($file);
            if ( $file =~ /LOD$/ ) {
                copy_files_to_filer( Node => $Node, dest_path => $fw_path, files_to_transfer => [$file] ) if ($file);
            }
        }
    } elsif ( defined $files ) {
        foreach my $file ( @{$files} ) {
            chomp($file);
            if ( $file =~ /LOD$/ ) {
                my $tmp = $files_path . "/" . $file;
                copy_files_to_filer( Node => $Node, dest_path => $fw_path, files_to_transfer => [$tmp] ) if ($tmp);
            }
        }
    } elsif ( defined $files_path ) {
        my @files;
        opendir( FH, "$files_path" ) || die("Cannot open directory");
        my @thefiles = readdir(FH);
        closedir(FH);
        grep { push( @files, $_ ) if ( $_ =~ /LOD$/ ) } @thefiles;
        logcomment( "The number of files to copy : " . scalar @files . " and the files are @files" );
        foreach my $file (@files) {
            chomp($file);
            my $temp = $files_path . "/" . $file;
            copy_files_to_filer( Node => $Node, dest_path => $fw_path, files_to_transfer => [$temp] ) if ($temp);
        }
    }
    logcomment("$FILER_C : Verifying whether the firmware files are successfully copied to filer directory");
    my $firmware_files = $Filer_apiset->execute_raw_command( command => "ls /mroot/etc/disk_fw" );
    chomp($firmware_files);
    my @total_files = split( /\s+/, $firmware_files );
    grep { push( @fw_files, $_ ) if ( $_ =~ /LOD$/ ) } @total_files;
    my @actual_disks_t_dup = @fw_files;
    my $disk_max_sup       = 3;
    my $str;

    while (@actual_disks_t_dup) {
        my $loop_no = 0;
        my @disk_list_q;
        while ( $loop_no < $disk_max_sup ) {
            push @disk_list_q, pop @actual_disks_t_dup;
            $loop_no++;
            last if ( !(@actual_disks_t_dup) );
        }
        my $local_str = join( " ", @disk_list_q );
        $str = "$str" . "$local_str" . "\n";
    }
    logcomment( "$FILER_C : Total number of firmware files present in filer directory are " . scalar(@fw_files) . " and the files are \n $str" );
    return @fw_files;
}

sub zns_check_drv_path {
    my (%opts) = validate( @_, { node_present => { type => ARRAYREF } } );
    my @Nodes;
    if ( defined( @{ $opts{node_present} } ) ) {
        @Nodes = @{ $opts{node_present} };
    } else {
        @Nodes = NACL::C::Node->find();
    }

    my @local_nodes = NACL::C::Node->find();
    my $FILER_A     = $local_nodes[0]->name();
    my $FILER_B     = $local_nodes[1]->name();
    foreach my $Node (@Nodes) {
        $Node->refresh_command_interface();
        my $FILER_C = $Node->name();
        logcomment("Checking Drive path on $FILER_C");
        my $Transit_obj = NACL::Transit->new( name => $FILER_C );

        #my $filer_state = $Transit_obj->get_state( 'timeout' => 3600, 'get_state_timeout' => 7200 );
        my $filer_state = $Transit_obj->get_state( 'timeout' => $timeout, 'get_state_timeout' => $timeout );
        if ( $filer_state =~ /CLI/gi ) {
            my $Api_Set_Obj = $Node->get_7m_or_nodescope_apiset();
            $Api_Set_Obj->execute_raw_command( 'command' => "\013" );
            $Api_Set_Obj->execute_raw_command( 'command' => "\013" );
            my $out = $Api_Set_Obj->execute_raw_command( 'command' => "set -rows 0;storage disk show -fields diskpathnames" );
            my @miss_drv;
            logcomment("FILER A : $FILER_A and FILER_B : $FILER_B");
 my $Transit_obj;
my $Transit_obj1;
    $Transit_obj = NACL::Transit->new( name => $FILER_A );
    my $filer_state = $Transit_obj->get_state( 'timeout' => 3600, 'get_state_timeout' => 3600 );
    $Transit_obj1 = NACL::Transit->new( name => $FILER_B );
    my $filer_state1 = $Transit_obj1->get_state( 'timeout' => 3600, 'get_state_timeout' => 3600 );
            logcomment("FILER A : $FILER_A and FILER_B : $FILER_B");
if($out !~/$FILER_A/ && $out =~/$FILER_B/ && $filer_state !~ /CLI|UP/ig){
logcomment("**FATAL** : Filer $FILER_A is DOWN . Hence exiting the test");
return 1;
}elsif($out !~/$FILER_B/ && $out =~/$FILER_A/ && $filer_state1 !~ /CLI|UP/ig){
logcomment("**FATAL** : Filer $FILER_B is DOWN . Hence exiting the test");
return 1;
}else{
            foreach my $line ( split /\n/, $out ) {
                my ( $drv, $path );
                next if ( $line =~ /disk|^-|displayed|debug|entries|^$|session\.change|\[\S+\]/ );
                if ( ( $line =~ /$FILER_A/ ) && ( $line =~ /$FILER_B/ ) ) {
                    my ( $drv, $path ) = $line =~ /(\S+)\s+(\S+)/;
                    my ($last_digit) = $drv =~ /\d+\.\d+\.(\d+)/;
                    if ( $path =~ /(\S+:\d\S+\.$last_digit){1}(\S+:\d\S\.$last_digit){1}/i && $path =~ /\S+\:\w+\.\d+\,/ ) {
                        logcomment("Drive $drv has path $path");
                    } elsif ( $path =~ /(\S+:\w+\.\d+\.\d+\.$last_digit\,){3}(\S+:\w+\.\d+\.\d+\.$last_digit){1}/i && $path =~ /\S+\:\w+\.\d+\.\d+\.\d+/ ) {
                        logcomment("Filer : $FILER : Drive $drv has path $path");
                    } else {
                        logcomment("Filer : $FILER : Drive $drv has path $path");
                        push( @miss_drv, $drv );
                    }
                } else {
                    my ( $drv, $path ) = $line =~ /(\S+)\s*\S*\:\S+/;
                    logcomment("Drive $drv path is incomplete");
                    push( @miss_drv, $drv );
                }

            }
}
            if (@miss_drv) {
logcomment("Drive path missing, collect telemetry logs");
                my @fail_drives = ();
                my @drives      = ();
                my $out1        = $Api_Set_Obj->execute_raw_command( 'command' => "run local disk_list" );
                foreach my $line ( split /\n/, $out1 ) {
                    if ( $line =~ /(\S+)\s*\S+/ ) {
                        my $drv = $1;
                        push( @drives, $drv );
                    }
                }
                foreach my $disk (@miss_drv) {
                    my $temp = $1 if ( $disk =~ /\S+\.\S+\.(\S+)/ );
                    foreach my $drv1 (@drives) {
                        my $temp1;
                        if ( $drv1 =~ /\S+\.\S+\.\S+\.(\S+)/ ) {
                            $temp1 = $1;
                        } elsif ( $drv1 =~ /\S+\.(\S+)/ ) {
                            $temp1 = $1;
                        }
                        if ( $temp =~ /^$temp1$/ ) {
                            push( @fail_drives, $drv1 );
                        }
                    }
                }
                foreach my $nv (@fail_drives) {
                    my $out1 = $Api_Set_Obj->execute_raw_command( 'command' => "run local disk dump_telemetry_log $nv" );
                    if ( $out1 =~ /Telemetry log successfully written at (\S+)/ ) {
                        logcomment("Telemetry log successfully written at $1");
                    }
                }
            }
if (@miss_drv) {
                logcomment("!!!! Drive path is MISSING for drives : @miss_drv");
                logcomment("**FATAL** : DRIVE Path is MISSING for drives : @miss_drv");
                return 1;
            }


            sleep 30;
        }

    }
    return 0;

}

1;

